from myffe.tools.color_printer import ColorPrinter

def inspect_data(data):
    """检查数据大小和行数
    
    Args:
        data: pandas DataFrame，输入数据
        
    Returns:
        int: Dask处理阈值（500万行）
    """
    # 检查数据类型
    data_type = type(data).__name__    #return the easy name to identify the data type
    ColorPrinter.print_info(f'数据类型: {data_type}')
    
    # 基本信息
    data_size = data.memory_usage(deep=True).sum() / (1024 * 1024)  # convert to MB
    ColorPrinter.print_info(f'数据大小: {data_size:.2f} MB')
    ColorPrinter.print_info(f'数据行数: {len(data)} 行')
    ColorPrinter.print_info(f'数据列数: {len(data.columns)} 列')
    
    # 列信息
    numeric_cols = data.select_dtypes(include=['number']).columns.tolist()
    categorical_cols = data.select_dtypes(include=['object', 'category']).columns.tolist()
    datetime_cols = data.select_dtypes(include=['datetime64']).columns.tolist()
    
    ColorPrinter.print_info(f'数值列数量: {len(numeric_cols)}')
    ColorPrinter.print_info(f'类别列数量: {len(categorical_cols)}')
    ColorPrinter.print_info(f'时间列数量: {len(datetime_cols)}')
    
    # 缺失值信息
    missing_values = data.isnull().sum().sum()
    total_cells = data.size
    missing_percentage = (missing_values / total_cells) * 100 if total_cells > 0 else 0
    
    if missing_values > 0:
        ColorPrinter.print_warning(f'缺失值数量: {missing_values} ({missing_percentage:.2f}%)')
    else:
        ColorPrinter.print_success('无缺失值')
    
    # 数据范围信息（仅数值列）
    if numeric_cols:
        numeric_data = data[numeric_cols]
        min_values = numeric_data.min()
        max_values = numeric_data.max()
        mean_values = numeric_data.mean()
        
        ColorPrinter.print_info('数值列基本统计:')
        for col in numeric_cols[:5]:  # 只显示前5个数值列
            try:
                ColorPrinter.print_info(f'  {col}: 最小值={min_values[col]:.2f}, 最大值={max_values[col]:.2f}, 均值={mean_values[col]:.2f}')
            except (TypeError, ValueError):
                # 处理无法格式化的情况
                ColorPrinter.print_info(f'  {col}: 最小值={min_values[col]}, 最大值={max_values[col]}, 均值={mean_values[col]}')
        if len(numeric_cols) > 5:
            ColorPrinter.print_info(f'  ... 还有 {len(numeric_cols) - 5} 个数值列')
    
    return None



import pandas as pd
import numpy as np
import time
# import the need modules
import sys
import os

# 添加MFM目录到路径
current_dir = os.path.dirname(os.path.abspath(__file__))
# py_tools -> route -> FFEv3_2 -> FFE -> MFM -> 根目录
root_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(current_dir)))))
if root_dir not in sys.path:
    sys.path.append(root_dir)

from myffe.tools.interactive import interactive
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.len_match import len_match
from myffe.tools.visualize_route import visualize_route
from myffe.tools.interactive_utils import get_interactive_choice
from myffe.tools.input_utils import ok_input

def analyze_nan_distribution(data, auto_test=False):
    """分析数据的缺失值分布，为每列推荐适合的缺失值处理方法
    
    Args:
        data: pandas DataFrame，输入数据
        auto_test: bool，是否自动测试模式，默认为 False
    
    Returns:
        dict: 包含缺失值处理参数的字典，使用嵌套字典列表结构
    """
    nan_params = {
        'nan_ways': []  # 存储每个列的处理参数
    }

    nan_missing_process_choice = ['drop_col_high_missing', 'fill_nan_knn', 'fill_nan_mean', 'fill_nan_mode', 'fill_nan_forward', 'fill_nan_backward', 'fill_nan_intelligent', 'fill_nan_ei', 'specify_fill', '随机填充','skip']
    # 计算每列的缺失值比例
    nan_ratio = data.isnull().mean() # 计算每列的缺失值比例
    nan_columns = nan_ratio[nan_ratio > 0].index.tolist() # 筛选出缺失值比例大于0的列
    
    if not nan_columns:
        ColorPrinter.print_info("数据中没有缺失值")
        return {'nan_preprocessing': 'skip'}
    
    
    inter_nan = get_interactive_choice(
        title="智能推荐？",
        description="选择'ok'将自动为所有缺失值列推荐最佳处理方法，选择'no'需要手动选择",
        auto_test=auto_test
    )
    
    if not inter_nan:
        selected_cols = interactive(nan_columns  , title="请按上下箭头选择，回车确认缺失值处理列,点击'm'键进行多选择", description="选择要处理的缺失值列，可以多选", min_select=1, max_select=len(nan_columns)).run()
        
        # 确保返回值是列表
        if selected_cols is None:
            time_cols = []
        elif isinstance(selected_cols, str):
            time_cols = [selected_cols]
        else:
            time_cols = selected_cols
        
        if time_cols:
            # 为每个选择的列单独设置处理方法和参数
            for col in time_cols:
                selected_method = interactive(nan_missing_process_choice,min_select = 1, max_select = 1, title=f"请按上下箭头选择，回车确认{col}的缺失值处理方法", description="drop_col_high_missing:删除缺失率高的列, fill_nan_knn:KNN填充, fill_nan_mean:均值填充, fill_nan_mode:众数填充, fill_nan_forward:前向填充, fill_nan_backward:后向填充, fill_nan_intelligent:智能填充, fill_nan_ei:集成填充, specify_fill:用户指定填充值, 随机填充:随机选择填充方式").run()
                # ensure the selected_method is a list
                method = selected_method[0]
                
                # 为当前列创建参数配置，包含所有可能的参数占位
                col_params = {
                    'col': col,
                    'method': method,
                    'drop_threshold': 0.5,  # 默认值
                    'knn_n_neighbors': 5,   # 默认值
                    'ei_model_type': 'linear',  # 默认值
                    'poly_degree': 2,  # 默认值
                    'fill_value': ''  # 默认值，使用空字符串代替 None
                }
                
                if method == '随机填充':
                    from random import choice
                    method = choice(nan_missing_process_choice)
                    col_params['method'] = method
                elif method == 'specify_fill':
                    # 让用户输入填充值
                    ColorPrinter.print_info(f'请为列 {col} 输入填充值：')
                    # 根据列的数据类型进行验证
                    col_dtype = data[col].dtype
                    fill_value = None
                    while fill_value is None:
                        if pd.api.types.is_numeric_dtype(col_dtype):
                            # 数值类型
                            if pd.api.types.is_integer_dtype(col_dtype):
                                fill_value = ok_input('int')
                            else:
                                fill_value = ok_input('float')
                        else:
                            # 非数值类型，使用字符串
                            fill_value = ok_input('str')
                    col_params['fill_value'] = fill_value
                elif method == 'fill_nan_ei':
                    # 为当前列选择EI模型类型
                    model_options = ['linear', 'ridge', 'lasso', 'random_forest', 'gradient_boosting', '自定义']
                    selected_model = interactive(model_options, min_select=1, max_select=1, title=f"请按上下箭头选择，回车确认列 {col} 的EI模型", description="选择EI方法中使用的机器学习模型").run()
                    # 确保返回值是单个值
                    model_type = selected_model[0]
                    if model_type == '自定义':
                        # 让用户输入自定义模型路径
                        ColorPrinter.print_info('请输入自定义模型完整路径（例如：sklearn.ensemble.RandomForestRegressor）：')
                        time.sleep(3)
                        custom_model = ok_input('str')
                        col_params['ei_model_type'] = custom_model
                        ColorPrinter.print_success(f'你已为列 {col} 选择自定义EI模型：' + custom_model)
                    else:
                        col_params['ei_model_type'] = model_type
                        ColorPrinter.print_success(f'你已为列 {col} 选择EI模型类型：' + model_type)
                    
                    # 为当前列选择多项式阶数
                    # 根据原始数据维度推荐多项式阶数
                    original_feature_count = len(data.columns)
                    data_rows = len(data)
                    
                    # 默认推荐阶数
                    default_degree = 2
                    if original_feature_count <= 5:
                        default_degree = 2
                    elif original_feature_count <= 10:
                        default_degree = 3
                    else:
                        default_degree = 4
                    
                    # 数据量特别大时可以适当增加阶数
                    if data_rows >= 5000:
                        default_degree = min(default_degree + 1, 5)
                    
                    # 构建选项，默认选项放在第一位
                    degree_options = [str(default_degree), '1', '2', '3', '4', '5', '自定义']
                    # 去重并保持顺序
                    degree_options = list(dict.fromkeys(degree_options))
                    
                    selected_degree = interactive(degree_options, min_select=1, max_select=1, title=f"请按上下箭头选择，回车确认列 {col} 的多项式特征阶数", description=f"设置EI方法中多项式回归的特征阶数，根据数据维度推荐：{default_degree}").run()
                    # 确保返回值是单个值
                    tem_degree = selected_degree[0]
                    if tem_degree == '自定义':
                        # 自动测试模式下，使用默认值
                        if auto_test:
                            degree = default_degree
                        else:
                            # 让用户输入自定义的多项式阶数
                            degree = ok_input('int', min_value=1, max_value=10)
                    else:
                        degree = int(tem_degree)
                    col_params['poly_degree'] = degree
                    ColorPrinter.print_success(f'你已为列 {col} 选择多项式特征阶数：' + str(degree))
                elif method == 'drop_col_high_missing':
                    threshold_options = ['0.3', '0.4', '0.5', '0.6', '0.7', '自定义']
                    selected_threshold = interactive(threshold_options, min_select=1, max_select=1, title=f"请按上下箭头选择，回车确认列 {col} 的高缺失值删除阈值", description="设置列的缺失值比例阈值，超过此比例的列将被删除").run()
                    # 确保返回值是单个值
                    tem_threshold = selected_threshold[0]
                    if tem_threshold == '自定义':
                        # 让用户输入自定义的阈值
                        drop_threshold = ok_input('float', min_value=0, max_value=1)
                    else:
                        drop_threshold = float(tem_threshold)
                    col_params['drop_threshold'] = drop_threshold
                    ColorPrinter.print_success(f'你已为列 {col} 选择高缺失值列删除阈值：' + str(drop_threshold))
                elif method == 'fill_nan_knn':
                    knn_options = ['3', '5', '7', '9', '10', '11', '13', '自定义']
                    selected_knn = interactive(knn_options, min_select=1, max_select=1, title=f"请按上下箭头选择，回车确认列 {col} 的KNN填充邻居数", description="设置KNN填充的邻居数量，选择'自定义'可输入任意值").run()
                    # 确保返回值是单个值
                    tem_knn = selected_knn[0]
                    if tem_knn == '自定义':
                        # 自动测试模式下，使用默认值 5
                        if auto_test:
                            knn_n_neighbors = 5
                        else:
                            # 让用户输入自定义的KNN邻居数
                            knn_n_neighbors = ok_input('int', min_value=1, max_value=50)
                    else:
                        knn_n_neighbors = int(tem_knn)
                    col_params['knn_n_neighbors'] = knn_n_neighbors
                    ColorPrinter.print_success(f'你已为列 {col} 选择KNN填充的邻居数：' + str(knn_n_neighbors))
                
                # 添加到nan_ways列表
                nan_params['nan_ways'].append(col_params)
                ColorPrinter.print_success(f'你已选择{col}的缺失值处理方法：{method}')
    else:
        # 只分析确实有缺失值的列
        for col in nan_columns:
            col_nan_ratio = nan_ratio[col]
            
            # 为当前列创建参数配置，包含所有可能的参数占位
            col_params = {
                'col': col,
                'drop_threshold': 0.5,  # 默认值
                'knn_n_neighbors': 5,   # 默认值
                'ei_model_type': 'linear',  # 默认值
                'poly_degree': 2,  # 默认值
                'fill_value': ''  # 默认值，使用空字符串代替 None
            }
            
            # 检查是否需要删除高缺失值列
            if col_nan_ratio > 0.5:
                col_params['method'] = 'drop_col_high_missing'
                col_params['drop_threshold'] = 0.5
            
            # 根据列的数据类型和缺失值比例推荐填充方法
            elif col_nan_ratio > 0:
                # 检查是否满足EI方法的推荐条件
                # 1. 计算整体缺失行比例
                total_missing_rows = data.isnull().any(axis=1).mean()
                # 2. 计算编码后的特征数量估计
                feature_count = 0
                for col_name in data.columns:
                    if col_name != col:  # 排除当前列
                        if pd.api.types.is_categorical_dtype(data[col_name]) or pd.api.types.is_object_dtype(data[col_name]):
                            # 分类列编码后特征数估计（假设平均每个分类列有10个类别）
                            feature_count += min(data[col_name].nunique(), 10)
                        elif pd.api.types.is_datetime64_any_dtype(data[col_name]):
                            # 时间列编码后特征数（年、月、日、小时等）
                            feature_count += 4
                        else:
                            # 数值列直接计数
                            feature_count += 1
                
                # 满足EI推荐条件
                # 1. 整体缺失行比例小于30%
                # 2. 编码后的特征数量不超过30个
                # 3. 目标列缺失比例在5%-40%之间
                # 4. 编码前的列数量达到3个以上
                # 5. 数据行数在1000行以上
                original_feature_count = len(data.columns)
                data_rows = len(data)
                if total_missing_rows < 0.3 and feature_count <= 30 and col_nan_ratio > 0.05 and col_nan_ratio < 0.4 and original_feature_count >= 3 and data_rows >= 1000:
                    col_params['method'] = 'fill_nan_ei'
                    # 根据数据特征推荐模型类型
                    # 1. 计算特征相关性，判断是否存在多重共线性
                    numeric_cols = [col_name for col_name in data.columns if col_name != col and pd.api.types.is_numeric_dtype(data[col_name])]
                    corr_matrix = None
                    if len(numeric_cols) >= 2:
                        try:
                            corr_matrix = data[numeric_cols].corr()
                        except:
                            pass
                    
                    # 2. 根据数据特征选择模型
                    model_type = 'linear'  # 默认模型
                    if corr_matrix is not None:
                        # 检查是否存在多重共线性（相关系数绝对值大于0.8）
                        high_corr = (abs(corr_matrix) > 0.8).sum().sum() - len(numeric_cols)  # 减去对角线元素
                        
                        # 检查是否存在非线性关系（通过特征与目标变量的关系）
                        has_nonlinearity = False
                        try:
                            # 检查每个数值特征与目标变量的关系
                            for num_col in numeric_cols:
                                if num_col != col:
                                    # 计算皮尔逊相关系数（线性关系）
                                    linear_corr = abs(data[num_col].corr(data[col]))
                                    # 计算斯皮尔曼相关系数（非线性关系）
                                    spearman_corr = abs(data[num_col].corr(data[col], method='spearman'))
                                    # 如果斯皮尔曼相关系数显著大于皮尔逊相关系数，说明存在非线性关系
                                    if spearman_corr > linear_corr + 0.2:
                                        has_nonlinearity = True
                                        break
                        except:
                            pass
                        
                        if high_corr > 0:
                            # 存在多重共线性，推荐使用Ridge回归
                            model_type = 'ridge'
                        elif len(numeric_cols) > 10:
                            # 特征数量较多，推荐使用Lasso回归进行特征选择
                            model_type = 'lasso'
                        elif has_nonlinearity and data_rows < 10000:
                            # 存在明显的非线性关系，且数据量适中，推荐使用梯度提升
                            model_type = 'gradient_boosting'
                        elif data_rows > 8000:
                            # 数据量很大，推荐使用随机森林
                            model_type = 'random_forest'
                        elif data_rows > 2000 and data_rows <= 8000:
                            # 数据量适中，推荐使用梯度提升
                            model_type = 'gradient_boosting'
                        else:
                            # 数据量较小，默认使用线性回归
                            model_type = 'linear'
                    else:
                        # 没有足够的数值特征，默认使用线性回归
                        model_type = 'linear'
                    
                    col_params['ei_model_type'] = model_type
                    
                    # 根据数据特征推荐多项式阶数
                    # 特征数量较少时使用较低阶数，特征数量较多时使用较高阶数
                    poly_degree = 2  # 默认阶数
                    if feature_count <= 10:
                        poly_degree = 2
                    elif feature_count <= 20:
                        poly_degree = 3
                    else:
                        poly_degree = 4
                    # 数据量特别大时可以适当增加阶数
                    if data_rows >= 5000:
                        poly_degree = min(poly_degree + 1, 5)
                    
                    col_params['poly_degree'] = poly_degree
                else:
                    # 根据列的数据类型推荐其他填充方法
                    if pd.api.types.is_numeric_dtype(data[col]):
                        if col_nan_ratio > 0.1: #当数值列缺失值比例大于0.1时，建议使用KNN填充
                            col_params['method'] = 'fill_nan_knn'
                            col_params['knn_n_neighbors'] = 5
                        else:
                            col_params['method'] = 'fill_nan_mean'
                    elif pd.api.types.is_categorical_dtype(data[col]) or pd.api.types.is_object_dtype(data[col]):
                        #当分类列缺失值比例大于0.1时，建议使用众数填充
                        col_params['method'] = 'fill_nan_mode'
                    elif pd.api.types.is_datetime64_any_dtype(data[col]):
                        # 检查日期时间列的单调性
                        is_increasing = data[col].dropna().is_monotonic_increasing
                        is_decreasing = data[col].dropna().is_monotonic_decreasing
                        if is_increasing:
                            # 递增数据建议使用向前填充
                            col_params['method'] = 'fill_nan_forward'
                        elif is_decreasing:
                            # 递减数据建议使用向后填充
                            col_params['method'] = 'fill_nan_backward'
                        else:
                            # 无明显单调性，默认使用向前填充
                            col_params['method'] = 'fill_nan_forward'
                    else:
                        col_params['method'] = 'fill_nan_intelligent'
            
            # 添加到nan_ways列表
            nan_params['nan_ways'].append(col_params)
    
    result = {'nan_preprocessing': nan_params}
    visualize_route(result)
    return result

def nan_py(data, auto_test=False):
    """Python实现的缺失值处理分析
    
    Args:
        data: pandas DataFrame，输入数据
        drop_threshold: 缺失值比例阈值，默认为0.5
        auto_test: bool，是否自动测试模式，默认为 False
    
    Returns:
        dict: 包含缺失值处理参数的字典
    """
    
    return analyze_nan_distribution(data, auto_test=auto_test)
if __name__ == "__main__":
    # 测试代码
    import pandas as pd
    import numpy as np
    
    # 创建测试数据
    np.random.seed(42)
    data = pd.DataFrame({
        'numeric_col1': np.random.normal(100, 10, 20),
        'numeric_col2': np.random.normal(50, 5, 20),
        'categorical_col': np.random.choice(['A', 'B', 'C'], 20),
        'datetime_col': pd.date_range('2020-01-01', periods=20)
    })
    
    # 随机添加缺失值
    for col in data.columns:
        if col != 'datetime_col':
            mask = np.random.choice([True, False], size=20, p=[0.2, 0.8])
            data.loc[mask, col] = np.nan
    
    ColorPrinter.print_info("测试数据:")
    ColorPrinter.print_any(data.head(), color='white')
    ColorPrinter.print_any("\n缺失值统计:", color='blue')
    ColorPrinter.print_any(data.isnull().sum(), color='white')
    
    # 测试缺失值处理
    ColorPrinter.print_info("\n测试缺失值处理:")
    result = nan_py(data)
    ColorPrinter.print_success("处理结果:")
    ColorPrinter.print_any(result, color='white')


"""
这个文件输入一个data
输出一个字典
格式是：
{
    'nan_preprocessing': {
        'nan_ways': [
            {
                'col': 'col1',
                'method': 'fill_nan_mean'
            },
            {
                'col': 'col2',
                'method': 'fill_nan_knn',
                'knn_n_neighbors': 5
            },
            {
                'col': 'col3',
                'method': 'fill_nan_ei',
                'ei_model_type': 'linear',
                'poly_degree': 2
            },
            {
                'col': 'col4',
                'method': 'drop_col_high_missing',
                'drop_threshold': 0.5
            },
            {
                'col': 'col5',
                'method': 'specify_fill',
                'fill_value': 0
            }
        ]
    }
}

method的取值有

    'drop_col_high_missing'   # 删除缺失值比例超过阈值的列
    'fill_nan_knn'   # 使用KNN填充缺失值
    'fill_nan_mean'   # 使用均值填充缺失值
    'fill_nan_mode'   # 使用众数填充缺失值
    'fill_nan_forward'   # 前向填充缺失值
    'fill_nan_backward'   # 后向填充缺失值
    'fill_nan_intelligent'   # 智能填充缺失值
    'fill_nan_ei'   # 集成填充缺失值
    'specify_fill'   # 用户指定填充值
    '随机填充'   # 随机选择填充方式
    'skip'   # 跳过缺失值处理

参数处理逻辑：
- 每个列的处理配置是独立的，包含在nan_ways列表中的一个字典里
- 无论方法是否需要特定参数，所有可能的参数都会在配置中进行占位
- 未被选中的参数会使用默认值进行占位，确保配置结构的一致性
- 这种设计使得配置结构更加统一，便于后续处理和维护
"""



import sys
import os

# 添加项目根目录到Python路径
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..', '..')))

import pandas as pd
import numpy as np
from myffe.tools.interactive import interactive
from myffe.tools.visualize_route import visualize_route
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.param_utils import convert_to_list, ensure_list



def analyze_string_distribution(data):
    """分析数据的字符串分布，推荐适合的字符串处理方法
    
    Args:
        data: pandas DataFrame，输入数据
    
    Returns:
        list: 包含每个字符串列处理参数的字典列表
        list: 字符串列列表
    """
    # 找出字符串列
    string_columns = []
    for col in data.columns:
        # 跳过数值列
        if pd.api.types.is_numeric_dtype(data[col]):
            continue
        # 检查是否是字符串列
        if data[col].dtype == 'object' or data[col].dtype == 'string' or str(data[col].dtype) == 'str':
            # 检查是否是纯字符串列（非数值）
            try:
                pd.to_numeric(data[col])
            except ValueError:
                string_columns.append(col)
    
    # 为每个字符串列生成单独的参数配置
    column_params_list = []
    
    if string_columns:
        for col in string_columns:
            col_data = data[col].astype(str)
            unique_count = col_data.nunique()
            # 处理可能的NaN值，确保所有值都是字符串
            lengths = col_data.apply(lambda x: len(x) if isinstance(x, str) else 0)
            col_avg_length = lengths.mean()
            sample_count = len(col_data)
            
            # 分析单个列的特征并推荐处理方法
            if unique_count < 10:
                col_choice = 'one_hot_encoder'  # 唯一值较少，使用独热编码
            elif unique_count < 100:
                col_choice = 'label_encoder'  # 唯一值适中，使用标签编码
            elif col_avg_length > 20:
                col_choice = 'tfidf'  # 长文本，使用TF-IDF
            else:
                col_choice = 'frequency_encoder'  # 唯一值较多，使用频率编码
            
            # 构建该列的参数配置
            col_params = {
                'str_choice': col_choice,
                'str_cols': col,  # 只包含当前列
                'tfidf_min_df': 1,
                'tfidf_max_df': 1.0,
                'tfidf_max_features': None,
                'lda_max_df': 1.0,
                'lda_min_df': 1,
                'lda_max_features': 1000,
                'lda_n_components': 10,
                'lda_random_state': 42,
                'use_chinese_tokenizer': False,
                'feature_selection': False,
                'k_features': 100
            }
            
            # 根据该列的特征调整参数
            if col_choice in ['tfidf', 'count']:
                # 根据唯一值数量调整特征数
                if unique_count > 1000:
                    col_params['tfidf_max_features'] = 1000
                elif unique_count > 500:
                    col_params['tfidf_max_features'] = 500
                
                # 根据样本数量调整最小文档频率
                if sample_count > 1000:
                    col_params['tfidf_min_df'] = 0.01
                elif sample_count > 100:
                    col_params['tfidf_min_df'] = 0.05
                
                # 根据样本数量调整最大文档频率
                if sample_count > 5000:
                    col_params['tfidf_max_df'] = 0.95
                elif sample_count > 1000:
                    col_params['tfidf_max_df'] = 0.98
            
            if col_choice == 'lda':
                # 根据唯一值数量调整主题数
                if unique_count > 1000:
                    col_params['lda_n_components'] = 20
                elif unique_count > 500:
                    col_params['lda_n_components'] = 15
                elif unique_count < 100:
                    col_params['lda_n_components'] = 5
                
                # 根据样本数量调整特征数
                if sample_count > 1000:
                    col_params['lda_max_features'] = 2000
                elif sample_count > 100:
                    col_params['lda_max_features'] = 1500
                
                # 根据样本数量调整最小文档频率
                if sample_count > 1000:
                    col_params['lda_min_df'] = 0.01
                elif sample_count > 100:
                    col_params['lda_min_df'] = 0.05
                
                # 根据样本数量调整最大文档频率
                if sample_count > 5000:
                    col_params['lda_max_df'] = 0.95
                elif sample_count > 1000:
                    col_params['lda_max_df'] = 0.98
                
                # 根据主题数调整随机种子
                col_params['lda_random_state'] = 42 + (col_params['lda_n_components'] % 10)
            
            # 自动检测是否需要中文分词
            has_chinese = False
            sample = col_data.sample(min(10, sample_count))
            for text in sample:
                # 确保text是字符串类型
                if isinstance(text, str):
                    if any('\u4e00' <= c <= '\u9fff' for c in text):
                        has_chinese = True
                        break
            
            if has_chinese:
                col_params['use_chinese_tokenizer'] = True
            
            # 根据文本长度决定是否进行特征选择
            if col_avg_length > 30:
                col_params['feature_selection'] = True
                # 根据唯一值数量和文本长度调整特征数量
                if unique_count > 500:
                    col_params['k_features'] = 300
                elif unique_count > 200:
                    col_params['k_features'] = 200
                else:
                    col_params['k_features'] = unique_count
            elif col_choice in ['tfidf', 'count', 'lda'] and unique_count > 500:
                # 对于高维特征，即使文本长度较短也考虑特征选择
                col_params['feature_selection'] = True
                col_params['k_features'] = 150 if unique_count > 300 else unique_count
            
            # 添加该列的分析信息
            analysis_info = {
                'unique_count': unique_count,
                'avg_length': col_avg_length,
                'sample_count': sample_count
            }
            
            # 为了显示分析信息，我们临时存储分析结果
            # 但在添加到参数列表前，我们创建一个不包含分析信息的副本
            col_params_copy = col_params.copy()
            col_params_copy['column_analysis'] = analysis_info
            column_params_list.append(col_params_copy)
    
    return column_params_list, string_columns

def get_string_columns(data):
    """获取数据中的字符串列
    
    Args:
        data: pandas DataFrame，输入数据
    
    Returns:
        list: 字符串列列表
    """
    string_columns = []
    for col in data.columns:
        # 跳过数值列
        if pd.api.types.is_numeric_dtype(data[col]):
            continue
        # 检查是否是字符串列
        if data[col].dtype == 'object' or data[col].dtype == 'string' or str(data[col].dtype) == 'str':
            # 检查是否是纯字符串列（非数值）
            try:
                pd.to_numeric(data[col])
            except ValueError:
                string_columns.append(col)
    return string_columns

def str_py(data, auto_test=False):
    """Python实现的字符串处理分析
    
    Args:
        data: pandas DataFrame，输入数据
        auto_test: bool，是否自动测试模式，默认为 False
    
    Returns:
        dict: 包含字符串处理参数的字典
    """
    # 首先获取字符串列
    string_columns = get_string_columns(data)
    
    if not string_columns:
        ColorPrinter.print_info("没有发现需要处理的字符串列")
        return 'skip'
    
    # 显示发现的字符串列信息
    ColorPrinter.print_info(f"发现 {len(string_columns)} 个字符串列需要处理")
    
    # 智能推荐菜单
    if auto_test:
        print("\033[31m下面进行测试使用智能推荐\033[0m")
        choice = "使用推荐方案"
    else:
        menu_options = [
            "使用推荐方案",
            "自定义处理方案",
            "跳过处理"
        ]
        menu = interactive(menu_options, title="字符串处理", description="请选择处理方式:(m键表示选择回车表示确认)" , min_select=1, max_select=1)
        
        selected = menu.run()
        selected_list = ensure_list(selected)
        choice = selected_list[0] if selected_list else "跳过处理"
    
    if choice == "跳过处理":
        ColorPrinter.print_info("跳过字符串处理")
        return 'skip'
    
    elif choice == "使用推荐方案":
        ColorPrinter.print_success("使用推荐方案")
        # 分析字符串分布并获取推荐参数
        column_params_list, _ = analyze_string_distribution(data)
        
        # 显示每个列的推荐处理方法
        ColorPrinter.print_info("每个字符串列的推荐处理方法:")
        for i, col_params in enumerate(column_params_list):
            col = col_params['str_cols']
            analysis = col_params['column_analysis']
            ColorPrinter.print_info(f"  {col}: {col_params['str_choice']} (唯一值: {analysis['unique_count']}, 平均长度: {analysis['avg_length']:.2f})")
        
        # 显示详细分析信息
        ColorPrinter.print_info("\n详细分析信息:")
        for i, col_params in enumerate(column_params_list):
            col = col_params['str_cols']
            analysis = col_params['column_analysis']
            ColorPrinter.print_info(f"  列 {col}:")
            ColorPrinter.print_info(f"    唯一值数量: {analysis['unique_count']} (不同值的数量，影响编码方法选择)")
            ColorPrinter.print_info(f"    平均长度: {analysis['avg_length']:.2f} (字符串平均长度，影响特征提取方法)")
            ColorPrinter.print_info(f"    样本数量: {analysis['sample_count']} (数据量大小，影响参数调优)")
            ColorPrinter.print_info(f"    中文分词: {col_params['use_chinese_tokenizer']} (是否使用中文分词器)")
            ColorPrinter.print_info(f"    特征选择: {col_params['feature_selection']} (是否进行特征选择)")
            if col_params['feature_selection']:
                ColorPrinter.print_info(f"    特征数量: {col_params['k_features']} (保留的特征数量)")
        
        # 构建结果，使用单个 str_preprocessing 字典包含所有列的处理参数
        result = {}
        str_preprocessing_params = {
            'str_ways': [],
            'tfidf_min_df': 1,
            'tfidf_max_df': 1.0,
            'tfidf_max_features': None,
            'lda_min_df': 1,
            'lda_max_df': 1.0,
            'lda_max_features': 1000,
            'lda_n_components': 10,
            'lda_random_state': 42
        }
        
        # 为每个列生成处理参数
        for i, col_params in enumerate(column_params_list):
            # 将 str_choice 改为 method 格式
            clean_params = col_params.copy()
            if 'column_analysis' in clean_params:
                del clean_params['column_analysis']
            # 将 str_choice 包装为 method 列表
            if 'str_choice' in clean_params:
                clean_params['method'] = [clean_params['str_choice']]
                del clean_params['str_choice']
            # 将每个列的参数添加到 str_ways 列表中
            str_preprocessing_params['str_ways'].append(clean_params)
        
        result['str_preprocessing'] = str_preprocessing_params
        
        # 可视化处理路线
        visualize_route(result)
        
        return result
    elif choice == "自定义处理方案":
        # 自定义处理方案
        ColorPrinter.print_info("自定义处理方案")
        
        # 1. 选择要处理的列
        col_menu = interactive(string_columns, title="选择列", description="请选择要处理的字符串列:", min_select=1, max_select=len(string_columns))
        selected_cols = col_menu.run()
        if not selected_cols:
            ColorPrinter.print_warning("未选择任何列，跳过处理")
            return 'skip'
        
        # 为每个选择的列单独选择处理方法和参数
        result = {}
        for i, col in enumerate(selected_cols):
            ColorPrinter.print_info(f"\n处理列: {col}")
            
            # 2. 为当前列选择处理方法
            method_options = [
                "label_encoder",
                "one_hot_encoder",
                "frequency_encoder",
                "count_encoder",
                "target_encoder",
                "tfidf",
                "count",
                "lda"
            ]
            method_menu = interactive(method_options, title=f"选择{col}的处理方法", description="请选择字符串处理方法:" , min_select=1, max_select=1)
            
            selected = method_menu.run()
            selected_list = ensure_list(selected)
            str_choice = selected_list[0] if selected_list else 'skip'
            
            # 3. 根据选择的处理方法，设置相应的参数
            col_params = {
                'str_choice': str_choice,
                'str_cols': col,  # 只包含当前列
                'tfidf_min_df': 1,
                'tfidf_max_df': 1.0,
                'tfidf_max_features': None,
                'lda_max_df': 1.0,
                'lda_min_df': 1,
                'lda_max_features': 1000,
                'lda_n_components': 10,
                'lda_random_state': 42,
                'use_chinese_tokenizer': False,
                'feature_selection': False,
                'k_features': 100
            }
            
            # 针对不同的处理方法，添加特定参数
            if str_choice in ['tfidf', 'count', 'lda']:
                # 分词器选择
                tokenizer_options = ["False", "True"]
                tokenizer_menu = interactive(tokenizer_options, title="分词器选择", description="请选择分词器: False=默认分词器(英文), True=中文分词器(jieba)")
                selected = tokenizer_menu.run()
                selected_list = ensure_list(selected)
                use_chinese_tokenizer = selected_list[0] if selected_list else "False"
                col_params['use_chinese_tokenizer'] = use_chinese_tokenizer == "True"
                
                # 特征选择
                feature_options = ["False", "True"]
                feature_menu = interactive(feature_options, title="特征选择", description="是否进行特征选择: False=不选择, True=使用卡方检验")
                selected = feature_menu.run()
                selected_list = ensure_list(selected)
                feature_selection = selected_list[0] if selected_list else "False"
                col_params['feature_selection'] = feature_selection == "True"
                
                if feature_selection == "True":
                    # 特征数量
                    k_features = input("请输入要保留的特征数量 (默认100): ")
                    col_params['k_features'] = int(k_features) if k_features else 100
            
            if str_choice in ['tfidf', 'count']:
                # TF-IDF/Count参数
                tfidf_min_df = input("请输入最小文档频率 (默认1): ")
                col_params['tfidf_min_df'] = float(tfidf_min_df) if tfidf_min_df else 1
                
                tfidf_max_df = input("请输入最大文档频率 (默认1.0): ")
                col_params['tfidf_max_df'] = float(tfidf_max_df) if tfidf_max_df else 1.0
                
                tfidf_max_features = input("请输入最大特征数 (默认None): ")
                col_params['tfidf_max_features'] = int(tfidf_max_features) if tfidf_max_features else None
            
            if str_choice == 'lda':
                # LDA参数
                lda_min_df = input("请输入最小文档频率 (默认1): ")
                col_params['lda_min_df'] = float(lda_min_df) if lda_min_df else 1
                
                lda_max_df = input("请输入最大文档频率 (默认1.0): ")
                col_params['lda_max_df'] = float(lda_max_df) if lda_max_df else 1.0
                
                lda_max_features = input("请输入最大特征数 (默认1000): ")
                col_params['lda_max_features'] = int(lda_max_features) if lda_max_features else 1000
                
                lda_n_components = input("请输入主题数 (默认10): ")
                col_params['lda_n_components'] = int(lda_n_components) if lda_n_components else 10
                
                lda_random_state = input("请输入随机种子 (默认42): ")
                col_params['lda_random_state'] = int(lda_random_state) if lda_random_state else 42
            
        # 构建结果，使用单个 str_preprocessing 字典包含所有列的处理参数
        str_preprocessing_params = {
            'str_ways': [],
            'tfidf_min_df': 1,
            'tfidf_max_df': 1.0,
            'tfidf_max_features': None,
            'lda_min_df': 1,
            'lda_max_df': 1.0,
            'lda_max_features': 1000,
            'lda_n_components': 10,
            'lda_random_state': 42
        }
        
        # 将每个列的参数添加到 str_ways 列表中，并转换格式
        for col, col_params in result.items():
            # 将 str_choice 改为 method 格式
            if 'str_choice' in col_params:
                col_params['method'] = [col_params['str_choice']]
                del col_params['str_choice']
            str_preprocessing_params['str_ways'].append(col_params)
        
        final_result = {'str_preprocessing': str_preprocessing_params}
        
        # 可视化处理路线
        visualize_route(final_result)
        
        return final_result


if __name__ == "__main__":
    """
    测试字符串处理功能
    """
    import pandas as pd
    import numpy as np
    
    print("=" * 80)
    print("字符串处理模块测试")
    print("=" * 80)
    
    # 创建测试数据
    data = pd.DataFrame({
        'category': ['A', 'B', 'C', 'A', 'B', 'C', 'A', 'B', 'C', 'A'],
        'short_text': ['apple', 'banana', 'orange', 'apple', 'banana', 'orange', 'apple', 'banana', 'orange', 'apple'],
        'long_text': [
            'This is a long text about machine learning and data science',
            'Natural language processing is a fascinating field of study',
            'Python is a popular programming language for data analysis',
            'Machine learning algorithms can learn from data patterns',
            'Deep learning models are used for image recognition',
            'Data visualization helps in understanding complex patterns',
            'Statistical analysis is important for data-driven decisions',
            'Feature engineering is crucial for machine learning models',
            'Model evaluation metrics help assess performance',
            'Hyperparameter tuning improves model accuracy'
        ],
        'chinese_text': [
            '这是一段中文文本',
            '机器学习是一个有趣的领域',
            '数据分析需要专业知识',
            '人工智能正在改变世界',
            'Python是一种流行的编程语言',
            '数据科学是一个快速发展的领域',
            '深度学习模型在图像识别中表现出色',
            '自然语言处理技术不断进步',
            '数据可视化帮助理解复杂数据',
            '统计分析是数据驱动决策的基础'
        ],
        'numeric_column': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    })
    
    print("测试数据:")
    print(data)
    print(f"\n数据形状: {data.shape}")
    print(f"列名: {list(data.columns)}")
    
    print("\n" + "=" * 80)
    print("测试字符串列检测")
    print("=" * 80)
    
    # 测试字符串列检测
    string_cols = get_string_columns(data)
    print(f"检测到的字符串列: {string_cols}")
    
    print("\n" + "=" * 80)
    print("测试字符串分布分析")
    print("=" * 80)
    
    # 测试字符串分布分析
    column_params_list, _ = analyze_string_distribution(data)
    for i, col_params in enumerate(column_params_list):
        col = col_params['str_cols']
        analysis = col_params['column_analysis']
        print(f"\n列 {col}:")
        print(f"  推荐处理方法: {col_params['str_choice']}")
        print(f"  唯一值数量: {analysis['unique_count']}")
        print(f"  平均长度: {analysis['avg_length']:.2f}")
        print(f"  样本数量: {analysis['sample_count']}")
        print(f"  中文分词: {col_params['use_chinese_tokenizer']}")
        print(f"  特征选择: {col_params['feature_selection']}")
        if col_params['feature_selection']:
            print(f"  特征数量: {col_params['k_features']}")
    
    print("\n" + "=" * 80)
    print("测试主处理函数")
    print("=" * 80)
    
    # 测试主处理函数
    try:
        result = str_py(data)
        print("\n处理结果:")
        if result == 'skip':
            print("跳过处理")
        else:
            for key, value in result.items():
                print(f"\n{key}:")
                print(f"  处理方法: {value['str_choice']}")
                print(f"  处理列: {value['str_cols']}")
                print(f"  中文分词: {value['use_chinese_tokenizer']}")
                print(f"  特征选择: {value['feature_selection']}")
    except Exception as e:
        print(f"测试过程中出错: {e}")
    
    print("\n" + "=" * 80)
    print("测试完成!")
    print("=" * 80)



import pandas as pd
import numpy as np

# 导入所需模块
import sys
import os

# 添加MFM目录到路径
current_dir = os.path.dirname(os.path.abspath(__file__))
# py_tools -> route -> FFEv3_2 -> FFE -> MFM -> 根目录
root_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(current_dir)))))
if root_dir not in sys.path:
    sys.path.append(root_dir)

from myffe.tools.interactive import interactive
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.visualize_route import visualize_route
from myffe.tools.interactive_utils import get_interactive_choice


def analyze_specify_needs(data):
    """分析数据，找出需要删除的列和行

    Args:
        data: pandas DataFrame，输入数据

    Returns:
        dict: 包含推荐删除列和行的字典
    """
    specify_params = {
        'drop_col': [],
        'drop_row': []
    }

    # 找出全为NaN的列
    all_nan_cols = data.columns[data.isna().all()].tolist()

    # 找出常量列（所有值都相同的列）
    constant_cols = []
    for col in data.columns:
        try:
            if data[col].nunique() == 1:
                constant_cols.append(col)
        except:
            pass

    # 合并推荐删除的列
    recommended_drop_col = list(set(all_nan_cols + constant_cols))

    # 找出全为NaN的行
    all_nan_rows = data.index[data.isna().all(axis=1)].tolist()

    specify_params['drop_col'] = recommended_drop_col
    specify_params['drop_row'] = all_nan_rows

    return specify_params


def display_missing_columns(data):
    """展示所有缺失列的信息

    Args:
        data: pandas DataFrame，输入数据

    Returns:
        list: 缺失列信息列表，格式为 [索引] 列名
    """
    missing_cols_info = []
    
    # 计算每列的缺失值数量和比例
    nan_counts = data.isna().sum()
    nan_ratios = data.isna().mean()
    
    # 筛选有缺失值的列
    missing_cols = nan_counts[nan_counts > 0]
    
    for idx, (col, nan_count) in enumerate(missing_cols.items()):
        nan_ratio = nan_ratios[col]
        info = f"[{idx}] {col} (缺失 {nan_count} 个值, 缺失率 {nan_ratio:.2%})"
        missing_cols_info.append(info)

    return missing_cols_info


def specify_py(data, auto_test=False):
    """Python实现的指定列删除分析
    
    Args:
        data: pandas DataFrame，输入数据
        auto_test: bool，是否自动测试模式，默认为 False
    
    Returns:
        dict: 包含指定删除参数的字典
    """
    # 展示缺失列信息
    ColorPrinter.print_info("=" * 60)
    ColorPrinter.print_info("数据缺失列信息：")
    ColorPrinter.print_info("=" * 60)

    missing_cols_info = display_missing_columns(data)

    if not missing_cols_info:
        ColorPrinter.print_info("数据中没有缺失列")
    else:
        # 展示缺失列信息
        for info in missing_cols_info:
            ColorPrinter.print_any(info, color='white')

    ColorPrinter.print_any("", color='white')

    # 对行进行智能推荐（自动删除全为NaN的行）
    recommend_params = analyze_specify_needs(data)
    drop_row = recommend_params['drop_row']

    if drop_row:
        ColorPrinter.print_success(f"智能推荐删除以下行（全为NaN的行）：{drop_row}")
    else:
        ColorPrinter.print_info("智能推荐：无需删除任何行")

    ColorPrinter.print_any("", color='white')

    # 询问是否使用智能推荐（针对列）
    inter_specify = get_interactive_choice(
        title="是否使用智能推荐列？",
        description="选择'ok'将自动推荐需要删除的列（全NaN列或常量列），选择'no'需要手动选择",
        auto_test=auto_test
    )

    drop_col = []

    if inter_specify:
        # 智能推荐模式（针对列）
        drop_col = recommend_params['drop_col']

        if drop_col:
            ColorPrinter.print_success(f"智能推荐删除以下列：{drop_col}")
        else:
            ColorPrinter.print_info("智能推荐：无需删除任何列")
    else:
        # 手动选择模式（只选择列）
        all_cols = list(data.columns)

        ColorPrinter.print_info("请选择要删除的列（按'm'键多选，按回车键确认）")
        selected_cols = interactive(all_cols, title="请按上下箭头选择，回车确认要删除的列,点击'm'键进行多选择", description="选择要删除的列，可以多选").run()

        drop_col = selected_cols

        ColorPrinter.print_success(f"你已选择删除以下列：{drop_col}")

    # 如果没有要删除的行和列，返回skip
    if not drop_col and not drop_row:
        ColorPrinter.print_info("没有需要删除的行和列")
        return 'skip'

    result = {
        'specify_preprocessing': {
            'specify_ways': [
                {
                    'params': {
                        'drop_col': drop_col,
                        'drop_row': drop_row
                    }
                }
            ]
        }
    }
    visualize_route(result)
    return result


if __name__ == "__main__":
    # 测试代码
    import pandas as pd
    import numpy as np

    # 创建测试数据
    np.random.seed(42)
    data = pd.DataFrame({
        'numeric_col1': np.random.normal(100, 10, 20),
        'numeric_col2': np.random.normal(50, 5, 20),
        'categorical_col': np.random.choice(['A', 'B', 'C'], 20),
        'constant_col': 'same_value',
        'nan_col': [np.nan] * 20
    })

    # 随机添加缺失值
    for col in data.columns:
        if col not in ['constant_col', 'nan_col']:
            mask = np.random.choice([True, False], size=20, p=[0.2, 0.8])
            data.loc[mask, col] = np.nan

    ColorPrinter.print_info("测试数据:")
    ColorPrinter.print_any(data.head(), color='white')
    ColorPrinter.print_any("\n缺失值统计:", color='blue')
    ColorPrinter.print_any(data.isnull().sum(), color='white')

    # 测试指定删除处理
    ColorPrinter.print_info("\n测试指定删除处理:")
    result = specify_py(data)
    ColorPrinter.print_success("处理结果:")
    ColorPrinter.print_any(result, color='white')



"""
Autoencoder Preprocessing

Implements autoencoder-based missing value imputation
"""

import numpy as np
import pandas as pd
from myffe.advance_preprocessing.preprocessing_all.autoencoder.autoencoder import AutoEncoder
from myffe.tools.color_printer import ColorPrinter


class AutoencoderPreprocessing:
    """
    Autoencoder Preprocessing for missing value imputation
    """
    
    def __init__(self, shortcut_autoencoder):
        """
        Initialize AutoencoderPreprocessing
        
        Args:
            shortcut_autoencoder (dict): Autoencoder configuration
        """
        self.shortcut_autoencoder = shortcut_autoencoder
        self.autoencoder = None
        
        if shortcut_autoencoder == 'skip':
            ColorPrinter.print_info("Autoencoder preprocessing skipped")
        else:
            ColorPrinter.print_info("Autoencoder preprocessing initialized")
            ColorPrinter.print_info(f"Autoencoder configuration: {shortcut_autoencoder}")
    
    def __call__(self, data, data_label=None):
        """
        Apply autoencoder preprocessing to data
        
        Args:
            data (DataFrame): Input data with missing values
            data_label (str): Label column name (if any)
            
        Returns:
            DataFrame: Data with missing values imputed
        """
        if self.shortcut_autoencoder == 'skip':
            ColorPrinter.print_info("Skipping autoencoder preprocessing")
            return data
        
        try:
            # Separate features and label if label is provided
            if data_label and data_label in data.columns:
                label_data = data[[data_label]].copy()
                feature_data = data.drop(columns=[data_label])
            else:
                label_data = None
                feature_data = data.copy()
            
            # Convert DataFrame to numpy array
            feature_np = feature_data.to_numpy()
            
            # Create autoencoder instance
            self.autoencoder = AutoEncoder(self.shortcut_autoencoder)
            
            # Apply autoencoder
            imputed_feature_np = self.autoencoder(feature_np)
            
            # Convert back to DataFrame
            imputed_feature_data = pd.DataFrame(imputed_feature_np, columns=feature_data.columns, index=feature_data.index)
            
            # Combine with label if it exists
            if label_data is not None:
                imputed_data = pd.concat([imputed_feature_data, label_data], axis=1)
            else:
                imputed_data = imputed_feature_data
            
            # Preserve original data types
            for col in data.columns:
                if data[col].dtype != imputed_data[col].dtype:
                    try:
                        imputed_data[col] = imputed_data[col].astype(data[col].dtype)
                    except:
                        pass
            
            ColorPrinter.print_success("OK Autoencoder preprocessing completed")
            return imputed_data
        except Exception as e:
            ColorPrinter.print_error(f"Error in autoencoder preprocessing: {str(e)}")
            import traceback
            traceback.print_exc()
            return data



import sys
import os
from abc import ABC

class interactive(ABC):    #Subclasses must implement the abstract method interactive (using the @abstractmethod decorator)
    def __init__(self, options ,title="Use arrow keys to select, press Enter to confirm", description="", min_select=0, max_select=None):
        super().__init__()
        self.options = self._check_options(options)
        self.title = title
        self.description = description
        self.current_idx = 0
        self.confirm_keys = ('\r', '\n')
        self.m = False
        self.min_select = min_select
        self.max_select = max_select if max_select is not None else len(options)
        # Initialize selected as empty list
        self.selected = []
        

    def _check_options(self, options):
        if not isinstance(options, list) or len(options) == 0:
            raise ValueError("Options list cannot be empty and must be of type list")
        if not all(isinstance(opt, str) for opt in options):
            raise TypeError("All elements in options list must be of type string")
        return options
# Multi-select functionality added: users can press 'm' to select current option and continue selecting other options until Enter is pressed to complete selection
    def _get_key(self):
        if sys.platform.startswith('win'):
            import msvcrt
            while True:
                if msvcrt.kbhit():
                    key = msvcrt.getch()
                    if key == b'\xe0':
                        key2 = msvcrt.getch()
                        if key2 == b'H':
                            return 'up'
                        elif key2 == b'P':
                            return 'down'
                    else:
                        try:
                            key_char = key.decode('ascii', errors='ignore').lower()
                            if key_char == 'm':
                                return 'm'
                            if key_char == 'r' or key_char == '\r':
                                return '\r'
                            return key_char
                        except:
                            return ''
        else:
            import termios
            import tty
            fd = sys.stdin.fileno()
            old_settings = termios.tcgetattr(fd)
            try:
                tty.setraw(fd)
                key = sys.stdin.read(1)
                if key == '\x1b':
                    sys.stdin.read(1)
                    key2 = sys.stdin.read(1)
                    if key2 == 'A':
                        return 'up'
                    elif key2 == 'B':
                        return 'down'
                return key.lower()
            finally:
                termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)

    def _clear_screen(self):
        os.system('cls' if sys.platform.startswith('win') else 'clear')

    def _print_menu(self):
        print(self.title)
        if self.description:
            print()
            print(f"Description: {self.description}")
        print()
        # Display selected options (below description)
        if self.selected:
            # Handle nested lists, ensure all elements are strings
            display_items = []
            for item in self.selected:
                if isinstance(item, list):
                    # Flatten nested list
                    for subitem in item:
                        display_items.append(str(subitem))
                else:
                    display_items.append(str(item))
            print("Selected: ", " | ".join(display_items))
        print(f"(Minimum selections: {self.min_select}, Maximum selections: {self.max_select})")
        print()
        for idx, opt in enumerate(self.options):
            if idx == self.current_idx:
                print(f"-> {opt}")
            else:
                print(f"   {opt}")

    def _handle_key(self, key):
        if key == 'up' and self.current_idx > 0:
            self.current_idx -= 1
        elif key == 'down' and self.current_idx < len(self.options) - 1:
            self.current_idx += 1
        elif key == 'm':
            self.m = True
            if not hasattr(self, 'selected'):
                self.selected = []
            current_option = self.options[self.current_idx]
            if current_option not in self.selected:
                self.selected.append(current_option)
            return 'm_select'
        elif key in ['\b', '\x08', 'backspace']:
            if hasattr(self, 'selected') and self.selected and len(self.selected) > 0:
                removed = self.selected.pop()
                print(f"\nRemoved: {removed}")
                return 'delete'
            return None
        elif key in self.confirm_keys:
            if not hasattr(self, 'selected'):
                self.selected = []
            if len(self.selected) >= self.min_select:
                return self.selected
            else:
                print(f"\nAt least {self.min_select} options must be selected, currently selected: {len(self.selected)}")
                return None

        return None

    def run(self):
        # 正常的交互式选择
        # Ensure selected is properly initialized as empty list
        self.selected = []
        self.m = False
        self.max_select = self.max_select if self.max_select is not None else len(self.options)
        
        # First display menu without clearing screen, wait for user to press Enter to start
        self._print_menu()
        print("\nPress Enter to start selection...")
        
        # Wait for user to press Enter
        while True:
            key = self._get_key()
            if key in self.confirm_keys:
                break
        
        # Start normal selection loop
        while True:
            self._clear_screen()
            self._print_menu()
            key = self._get_key()
            result = self._handle_key(key)
            if result == 'delete':
                continue
            if result == 'm_select':
                print(f"\nSelected: {self.selected[-1]}")
                print(f"Currently selected: {self.selected}")
                print("Continue selecting other options...")
                continue
            if isinstance(result, list):
                return result
            # Only process non-enter and non-delete keys, and ensure result is not a list
            if result is not None and not isinstance(result, list) and result not in self.confirm_keys and result != 'delete':
                if len(self.selected) >= self.max_select:
                    print(f"\nMaximum {self.max_select} options can be selected")
                    continue
                if result not in self.selected:
                    self.selected.append(result)
                print(f"\nSelected: {result}")
                print(f"Currently selected: {self.selected}")
                print("Continue selecting other options...")

# Multi-select functionality added here
if __name__ == "__main__":
    test_options = ["Feature Engineering", "Model Training", "Result Visualization", "Exit"]
    selector = interactive(test_options)
    selector.run()



from myffe.tools.color_printer import ColorPrinter


def len_match(list1, list2):
    if len(list1) != len(list2):
        ColorPrinter.print_error(f'错误: 列数量 ({len(list1)}) 与方法数量 ({len(list2)}) 不匹配')
        return False
    return True



# 预定义特征工程路线
# 用户可以通过传入数字来选择不同的预定义路线

import sys
import os
import pandas as pd
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.interactive import interactive
from myffe.tools.interactive_utils import get_interactive_choice

# 添加MFM目录到路径
current_dir = os.path.dirname(os.path.abspath(__file__))
route_dir = os.path.dirname(current_dir)
ffe_dir = os.path.dirname(route_dir)
mfm_dir = os.path.dirname(os.path.dirname(ffe_dir))  # 向上两级目录，得到 D:\HP\pythonwork\MFM
if mfm_dir not in sys.path:
    sys.path.append(mfm_dir)

# 全局routes字典，避免重复定义
ROUTES = {
    1: {
        "name": "基础路线",
        "description": "适合大多数数据集的基础预处理流程",
        "tools": ["缺失值处理", "异常值处理", "数据标准化"],
        "suitable_for": "通用数据集，快速处理"
    },
    2: {
        "name": "股票数据路线",
        "description": "专为股票数据优化的预处理流程",
        "tools": ["缺失值处理", "异常值处理", "时间处理", "特征工程", "数据标准化"],
        "suitable_for": "股票价格、交易量等金融时间序列数据"
    },
    3: {
        "name": "文本数据路线",
        "description": "包含文本处理的预处理流程",
        "tools": ["缺失值处理", "文本处理", "数据标准化"],
        "suitable_for": "包含文本特征的数据集"
    },
    4: {
        "name": "时间序列路线",
        "description": "强化时间特征提取的预处理流程",
        "tools": ["缺失值处理", "时间处理", "特征工程", "数据标准化"],
        "suitable_for": "时间序列预测任务"
    },
    5: {
        "name": "完整路线",
        "description": "包含所有预处理步骤的完整流程",
        "tools": ["缺失值处理", "异常值处理", "时间处理", "特征工程", "数据标准化", "数据归一化"],
        "suitable_for": "需要全面处理的复杂数据集"
    },
    6: {
        "name": "轻量级路线",
        "description": "快速处理小数据集的轻量级流程",
        "tools": ["缺失值处理", "数据标准化"],
        "suitable_for": "小数据集，快速原型开发"
    },
    7: {
        "name": "异常检测路线",
        "description": "专注于异常值检测和处理的流程",
        "tools": ["缺失值处理", "异常值处理", "数据标准化"],
        "suitable_for": "需要严格异常值处理的数据集"
    },
    8: {
        "name": "特征增强路线",
        "description": "强化特征工程的预处理流程",
        "tools": ["缺失值处理", "时间处理", "特征工程", "数据标准化", "数据归一化"],
        "suitable_for": "需要丰富特征的复杂预测任务"
    },
    9: {
        "name": "分类数据路线",
        "description": "适合分类任务的预处理流程",
        "tools": ["缺失值处理", "异常值处理", "文本处理", "特征工程", "数据标准化"],
        "suitable_for": "分类任务数据集"
    },
    10: {
        "name": "回归数据路线",
        "description": "适合回归任务的预处理流程",
        "tools": ["缺失值处理", "异常值处理", "时间处理", "特征工程", "数据标准化", "数据归一化"],
        "suitable_for": "回归任务数据集"
    },
    11: {
        "name": "中文文本路线",
        "description": "专为中文文本数据优化的预处理流程",
        "tools": ["缺失值处理", "文本处理（中文分词）", "特征选择", "数据标准化"],
        "suitable_for": "包含中文文本的数据集，如新闻、评论等"
    },
    12: {
        "name": "不平衡数据路线",
        "description": "处理类别不平衡的数据集的预处理流程",
        "tools": ["缺失值处理", "异常值处理", "样本处理（SMOTE）", "数据标准化"],
        "suitable_for": "类别不平衡的分类数据集"
    },
    13: {
        "name": "高维数据路线",
        "description": "处理特征维度较高的数据集的预处理流程",
        "tools": ["缺失值处理", "特征工程（降维）", "数据标准化"],
        "suitable_for": "高维数据集，如基因表达数据、图像特征等"
    },
    14: {
        "name": "金融数据路线",
        "description": "专为金融数据优化的预处理流程",
        "tools": ["缺失值处理（前向填充）", "异常值处理（Isolation Forest）", "时间处理", "特征工程", "数据标准化"],
        "suitable_for": "金融时间序列数据，如股票、期货、外汇等"
    },
    15: {
        "name": "医疗数据路线",
        "description": "适合医疗数据集的预处理流程",
        "tools": ["缺失值处理（智能填充）", "异常值处理", "特征工程", "数据标准化（RobustScaler）"],
        "suitable_for": "医疗健康数据集，如患者记录、医学影像特征等"
    },
    16: {
        "name": "电商数据路线",
        "description": "专为电商数据优化的预处理流程",
        "tools": ["缺失值处理（众数填充）", "文本处理（产品描述）", "时间处理（购买时间）", "数据标准化（Min-Max）"],
        "suitable_for": "电商交易数据，如用户购买记录、产品信息等"
    },
    17: {
        "name": "社交媒体数据路线",
        "description": "处理社交媒体数据的预处理流程",
        "tools": ["缺失值处理", "文本处理（社交媒体文本）", "时间处理（发布时间）", "数据标准化"],
        "suitable_for": "社交媒体数据，如微博、微信、Twitter等"
    },
    18: {
        "name": "传感器数据路线",
        "description": "处理传感器数据的预处理流程",
        "tools": ["缺失值处理（前向填充）", "异常值处理（多变量检测）", "时间处理", "特征工程（降维）", "数据标准化"],
        "suitable_for": "传感器采集的数据，如工业传感器、IoT设备数据等"
    },
    19: {
        "name": "多模态数据路线",
        "description": "处理包含多种类型数据的数据集的预处理流程",
        "tools": ["缺失值处理（智能填充）", "文本处理", "时间处理", "特征工程", "数据标准化", "数据归一化"],
        "suitable_for": "包含多种数据类型的数据集，如文本+图像+时间序列等"
    },
    20: {
        "name": "实时数据路线",
        "description": "适合实时数据流处理的预处理流程",
        "tools": ["缺失值处理（快速）", "异常值处理（快速）", "时间处理（基础）", "数据标准化"],
        "suitable_for": "实时数据流，如在线推荐系统、实时监控数据等"
    }
}


def get_route_params(route_number, data=None, auto_test=False):
    """
    根据路线编号获取预定义的特征工程参数
    
    Args:
        route_number (int): 路线编号
        1: 基础路线 - 适合大多数数据集
        2: 股票数据路线 - 专为股票数据优化
        3: 文本数据路线 - 包含文本处理
        4: 时间序列路线 - 强化时间特征提取
        5: 完整路线 - 包含所有预处理步骤
        6: 轻量级路线 - 快速处理小数据集
        7: 异常检测路线 - 专注于异常值处理
        8: 特征增强路线 - 强化特征工程
        9: 分类数据路线 - 适合分类任务
        10: 回归数据路线 - 适合回归任务
        11: 中文文本路线 - 专为中文文本数据优化
        12: 不平衡数据路线 - 处理类别不平衡的数据集
        13: 高维数据路线 - 处理特征维度较高的数据集
        14: 金融数据路线 - 专为金融数据优化
        15: 医疗数据路线 - 适合医疗数据集的处理
        16: 电商数据路线 - 专为电商数据优化
        17: 社交媒体数据路线 - 处理社交媒体数据
        18: 传感器数据路线 - 处理传感器数据
        19: 多模态数据路线 - 处理包含多种类型数据的数据集
        20: 实时数据路线 - 适合实时数据流处理
        data (DataFrame, optional): 输入数据集，用于自动检测列信息
        auto_test (bool): 是否为自动测试模式，为True时自动选择所有列
        data (DataFrame, optional): 输入数据集，用于自动检测列信息
    
    Returns:
        dict: 预定义的参数配置
    """
    # 配置前展示所有路线
    ColorPrinter.print_info("配置前 - 所有可用路线:")
    show_routes_visual()
    
    if route_number == 1:
        # 路线1: 基础路线
        ColorPrinter.print_info("选择了路线1: 基础路线 - 适合大多数数据集")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'thres_sigma_auto',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'drop',
                'contamination': 0.1,
                'outlier_replace_method': None
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 2:
        # 路线2: 股票数据路线
        ColorPrinter.print_info("选择了路线2: 股票数据路线 - 专为股票数据优化")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'thres_sigma_auto',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'drop',
                'contamination': 0.1,
                'outlier_replace_method': None
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 5,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 3:
        # 路线3: 文本数据路线
        ColorPrinter.print_info("选择了路线3: 文本数据路线 - 包含文本处理")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'str_preprocessing': {
                'str_ways': [{
                    'method': ['tfidf'],
                    'str_cols': 'all',
                    'tfidf_min_df': 0.01,
                    'tfidf_max_df': 0.95,
                    'tfidf_max_features': 1000,
                    'use_chinese_tokenizer': False,
                    'feature_selection': False,
                    'k_features': 100
                }]
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 4:
        # 路线4: 时间序列路线
        ColorPrinter.print_info("选择了路线4: 时间序列路线 - 强化时间特征提取")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 3,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 5:
        # 路线5: 完整路线
        ColorPrinter.print_info("选择了路线5: 完整路线 - 包含所有预处理步骤")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'thres_sigma_auto',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'drop',
                'contamination': 0.1,
                'outlier_replace_method': None
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 3,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            },
            'normalized_preprocessing': {
                'normalized_ways': [
                    {
                        'col': 'all',
                        'method': 'L2归一化'
                    }
                ]
            },
            'str_preprocessing': {
                'str_ways': []
            },
            'sample_preprocessing': {
                'sample_ways': []
            },
            'specify_preprocessing': {
                'specify_ways': []
            }
        }
        
    elif route_number == 6:
        # 路线6: 轻量级路线
        ColorPrinter.print_info("选择了路线6: 轻量级路线 - 快速处理小数据集")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 7:
        # 路线7: 异常检测路线
        ColorPrinter.print_info("选择了路线7: 异常检测路线 - 专注于异常值处理")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'thres_sigma_auto',
                'row_drop_thres': 2.5,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'drop',
                'contamination': 0.1,
                'outlier_replace_method': None
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 8:
        # 路线8: 特征增强路线
        ColorPrinter.print_info("选择了路线8: 特征增强路线 - 强化特征工程")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.85,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 4,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            },
            'normalized_preprocessing': {
                'normalized_ways': [
                    {
                        'col': 'all',
                        'method': 'L2归一化'
                    }
                ]
            }
        }
        
    elif route_number == 9:
        # 路线9: 分类数据路线
        ColorPrinter.print_info("选择了路线9: 分类数据路线 - 适合分类任务")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'thres_sigma_auto',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'drop',
                'contamination': 0.1,
                'outlier_replace_method': None
            },
            'str_preprocessing': {
                'str_ways': [{
                    'method': ['tfidf'],
                    'str_cols': 'all',
                    'tfidf_min_df': 0.01,
                    'tfidf_max_df': 0.95,
                    'tfidf_max_features': 1000,
                    'use_chinese_tokenizer': False,
                    'feature_selection': False,
                    'k_features': 100
                }]
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 5,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 10:
        # 路线10: 回归数据路线
        ColorPrinter.print_info("选择了路线10: 回归数据路线 - 适合回归任务")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'thres_sigma_auto',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'drop',
                'contamination': 0.1,
                'outlier_replace_method': None
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 3,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            },
            'normalized_preprocessing': {
                'normalized_ways': [
                    {
                        'col': 'all',
                        'method': 'L2归一化'
                    }
                ]
            }
        }
        
    elif route_number == 11:
        # 路线11: 中文文本路线
        ColorPrinter.print_info("选择了路线11: 中文文本路线 - 专为中文文本数据优化")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'str_preprocessing': {
                'str_ways': [{
                    'method': ['tfidf'],
                    'str_cols': 'all',
                    'tfidf_min_df': 0.01,
                    'tfidf_max_df': 0.95,
                    'tfidf_max_features': 2000,
                    'use_chinese_tokenizer': True,
                    'feature_selection': True,
                    'k_features': 1000
                }]
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 12:
        # 路线12: 不平衡数据路线
        ColorPrinter.print_info("选择了路线12: 不平衡数据路线 - 处理类别不平衡的数据集")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'thres_sigma_auto',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'drop',
                'contamination': 0.1,
                'outlier_replace_method': None
            },
            'sample_preprocessing': {
                'sample_ways': [
                    {
                        'mode': 'advance',
                        'method': ['MY_high_sample'],
                        'balance_method': 'oversample',
                        'random_state': 42,
                        'high_sample_selection': 'BorderlineSMOTE'
                    }
                ]
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 13:
        # 路线13: 高维数据路线
        ColorPrinter.print_info("选择了路线13: 高维数据路线 - 处理特征维度较高的数据集")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 50,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 14:
        # 路线14: 金融数据路线
        ColorPrinter.print_info("选择了路线14: 金融数据路线 - 专为金融数据优化")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_forward',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'isolation_forest',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'replace',
                'contamination': 0.1,
                'outlier_replace_method': 'median'
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 3,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 15:
        # 路线15: 医疗数据路线
        ColorPrinter.print_info("选择了路线15: 医疗数据路线 - 适合医疗数据集的处理")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_intelligent',  # 智能填充适合医疗数据
                        'drop_threshold': 0.3,  # 更严格的缺失值阈值
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'boxplot',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'replace',
                'contamination': 0.1,
                'outlier_replace_method': 'median'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.85,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 5,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'RobustScaler标准化'  # 对异常值不敏感
                    }
                ]
            }
        }
        
    elif route_number == 16:
        # 路线16: 电商数据路线
        ColorPrinter.print_info("选择了路线16: 电商数据路线 - 专为电商数据优化")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mode',  # 众数填充适合分类数据
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'str_preprocessing': {
                'str_ways': [{
                    'method': ['tfidf'],
                    'str_cols': 'all',
                    'tfidf_min_df': 0.001,
                    'tfidf_max_df': 0.9,
                    'tfidf_max_features': 1500,
                    'use_chinese_tokenizer': True,
                    'feature_selection': False,
                    'k_features': 100
                }]
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Min-Max标准化'  # 缩放到[0,1]范围
                    }
                ]
            }
        }
        
    elif route_number == 17:
        # 路线17: 社交媒体数据路线
        ColorPrinter.print_info("选择了路线17: 社交媒体数据路线 - 处理社交媒体数据")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mode',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'str_preprocessing': {
                'str_ways': [{
                    'method': ['tfidf'],
                    'str_cols': 'all',
                    'tfidf_min_df': 0.001,
                    'tfidf_max_df': 0.8,
                    'tfidf_max_features': 3000,
                    'use_chinese_tokenizer': True,
                    'feature_selection': True,
                    'k_features': 1500
                }]
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 18:
        # 路线18: 传感器数据路线
        ColorPrinter.print_info("选择了路线18: 传感器数据路线 - 处理传感器数据")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_forward',  # 前向填充适合时间序列传感器数据
                        'drop_threshold': 0.2,  # 更严格的缺失值阈值
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'multivariate',  # 多变量异常检测适合传感器数据
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'replace',
                'contamination': 0.1,
                'outlier_replace_method': 'median'
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 10,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    elif route_number == 19:
        # 路线19: 多模态数据路线
        ColorPrinter.print_info("选择了路线19: 多模态数据路线 - 处理包含多种类型数据的数据集")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_intelligent',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'str_preprocessing': {
                'str_ways': [{
                    'method': ['tfidf'],
                    'str_cols': 'all',
                    'tfidf_min_df': 0.01,
                    'tfidf_max_df': 0.9,
                    'tfidf_max_features': 2000,
                    'use_chinese_tokenizer': True,
                    'feature_selection': True,
                    'k_features': 1000
                }]
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'advance'
            },
            'relative_preprocessing': {
                'linear_cols': 'all',
                'linear_threshold': 0.9,
                'feature_cols': 'all',
                'feature_threshold': 0.5,
                'relation_func': 'func1',
                'pca_cols': 'all',
                'n_components': 50,
                'kernel_selection': 'pca',
                'n_features': 10
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            },
            'normalized_preprocessing': {
                'normalized_ways': [
                    {
                        'col': 'all',
                        'method': 'L2归一化'
                    }
                ]
            }
        }
        
    elif route_number == 20:
        # 路线20: 实时数据路线
        ColorPrinter.print_info("选择了路线20: 实时数据路线 - 适合实时数据流处理")
        
        params = {
            'nan_preprocessing': {
                'nan_ways': [
                    {
                        'col': 'all',
                        'method': 'fill_nan_mean',
                        'drop_threshold': 0.5,
                        'knn_n_neighbors': 5
                    }
                ]
            },
            'outlier_preprocessing': {
                'outlier_choice': 'thres_sigma_auto',
                'row_drop_thres': 3.0,
                'auto_drop_row': True,
                'specify_auto_drop_row_extr': 'auto',
                'outlier_action': 'drop',
                'contamination': 0.1,
                'outlier_replace_method': None
            },
            'time_preprocessing': {
                'time_col': 'auto',
                'time_type': 'basic'  # 基础时间特征，速度更快
            },
            'standard_preprocessing': {
                'standard_ways': [
                    {
                        'col': 'all',
                        'method': 'Z-score标准化'
                    }
                ]
            }
        }
        
    else:
        ColorPrinter.print_error(f"错误: 无效的路线编号 {route_number}")
        ColorPrinter.print_error("请选择以下路线之一:")
        ColorPrinter.print_error("1: 基础路线 - 适合大多数数据集")
        ColorPrinter.print_error("2: 股票数据路线 - 专为股票数据优化")
        ColorPrinter.print_error("3: 文本数据路线 - 包含文本处理")
        ColorPrinter.print_error("4: 时间序列路线 - 强化时间特征提取")
        ColorPrinter.print_error("5: 完整路线 - 包含所有预处理步骤")
        ColorPrinter.print_error("6: 轻量级路线 - 快速处理小数据集")
        ColorPrinter.print_error("7: 异常检测路线 - 专注于异常值处理")
        ColorPrinter.print_error("8: 特征增强路线 - 强化特征工程")
        ColorPrinter.print_error("9: 分类数据路线 - 适合分类任务")
        ColorPrinter.print_error("10: 回归数据路线 - 适合回归任务")
        ColorPrinter.print_error("11: 中文文本路线 - 专为中文文本数据优化")
        ColorPrinter.print_error("12: 不平衡数据路线 - 处理类别不平衡的数据集")
        ColorPrinter.print_error("13: 高维数据路线 - 处理特征维度较高的数据集")
        ColorPrinter.print_error("14: 金融数据路线 - 专为金融数据优化")
        ColorPrinter.print_error("15: 医疗数据路线 - 适合医疗数据集的处理")
        ColorPrinter.print_error("16: 电商数据路线 - 专为电商数据优化")
        ColorPrinter.print_error("17: 社交媒体数据路线 - 处理社交媒体数据")
        ColorPrinter.print_error("18: 传感器数据路线 - 处理传感器数据")
        ColorPrinter.print_error("19: 多模态数据路线 - 包含多种类型数据的数据集")
        ColorPrinter.print_error("20: 实时数据路线 - 适合实时数据流处理")
        return None
    
    # 配置后展示用户选择的路线
    ColorPrinter.print_success("\n参数配置完成！")
    ColorPrinter.print_info("配置的预处理工具:")
    for tool_name in params.keys():
        ColorPrinter.print_info(f"- {tool_name}")
    
    # 如果提供了数据，使用Interactive让用户选择每个工具要处理的列
    if data is not None:
        # 获取所有列名
        all_cols = list(data.columns)
        # 获取数值列
        numeric_cols = [col for col in all_cols if data[col].dtype in ['int64', 'float64']]
        # 获取字符串列
        string_cols = [col for col in all_cols if data[col].dtype == 'object']
        
        # 处理nan_preprocessing - 让用户选择要填充缺失值的列
        if 'nan_preprocessing' in params:
            if 'nan_ways' in params['nan_preprocessing']:
                for way in params['nan_preprocessing']['nan_ways']:
                    if way.get('col') == 'all':
                        if auto_test:
                            # 自动测试模式，选择所有数值列
                            if numeric_cols:
                                way['col'] = numeric_cols
                                ColorPrinter.print_info(f"自动测试模式: 选择所有数值列进行缺失值处理: {numeric_cols}")
                        else:
                            ColorPrinter.print_info("\n=== 缺失值处理 ===")
                            ColorPrinter.print_info("当前方法: " + way.get('method', 'fill_nan_mean'))
                            if numeric_cols:
                                inter = interactive(
                                    options=numeric_cols,
                                    title="选择要处理缺失值的列（按m多选）",
                                    description="选择要填充缺失值的列，按m选择多个，按回车确认",
                                    min_select=1,
                                    max_select=len(numeric_cols)
                                )
                                selected_cols = inter.run()
                                if selected_cols:
                                    way['col'] = selected_cols
        
        # 处理outlier_preprocessing - 让用户选择要检测异常值的列
        if 'outlier_preprocessing' in params:
            if numeric_cols:
                if auto_test:
                    # 自动测试模式，选择所有数值列
                    params['outlier_preprocessing']['outlier_cols'] = numeric_cols
                    ColorPrinter.print_info(f"自动测试模式: 选择所有数值列进行异常值检测: {numeric_cols}")
                else:
                    ColorPrinter.print_info("\n=== 异常值处理 ===")
                    ColorPrinter.print_info("当前方法: " + params['outlier_preprocessing'].get('outlier_choice', 'thres_sigma_auto'))
                    inter = interactive(
                        options=numeric_cols,
                        title="选择要检测异常值的列（按m多选）",
                        description="选择要检测异常值的列，按m选择多个，按回车确认",
                        min_select=1,
                        max_select=len(numeric_cols)
                    )
                    selected_cols = inter.run()
                    if selected_cols:
                        params['outlier_preprocessing']['outlier_cols'] = selected_cols
        
        # 处理str_preprocessing - 让用户选择要进行字符串编码的列
        if 'str_preprocessing' in params:
            if 'str_ways' in params['str_preprocessing']:
                for way in params['str_preprocessing']['str_ways']:
                    if way.get('str_cols') == 'all' and string_cols:
                        if auto_test:
                            # 自动测试模式，选择所有字符串列
                            way['str_cols'] = string_cols
                            ColorPrinter.print_info(f"自动测试模式: 选择所有字符串列进行字符串处理: {string_cols}")
                        else:
                            ColorPrinter.print_info("\n=== 字符串处理 ===")
                            ColorPrinter.print_info("当前方法: " + str(way.get('method', ['tfidf'])))
                            inter = interactive(
                                options=string_cols,
                                title="选择要进行字符串处理的列（按m多选）",
                                description="选择要编码的字符串列，按m选择多个，按回车确认",
                                min_select=1,
                                max_select=len(string_cols)
                            )
                            selected_cols = inter.run()
                            if selected_cols:
                                way['str_cols'] = selected_cols
        
        # 处理standard_preprocessing - 让用户选择要标准化的列
        if 'standard_preprocessing' in params:
            if 'standard_ways' in params['standard_preprocessing']:
                for way in params['standard_preprocessing']['standard_ways']:
                    if way.get('col') == 'all':
                        if numeric_cols:
                            if auto_test:
                                # 自动测试模式，选择所有数值列
                                way['col'] = numeric_cols
                                ColorPrinter.print_info(f"自动测试模式: 选择所有数值列进行标准化处理: {numeric_cols}")
                            else:
                                ColorPrinter.print_info("\n=== 标准化处理 ===")
                                ColorPrinter.print_info("当前方法: " + way.get('method', 'Z-score标准化'))
                                inter = interactive(
                                    options=numeric_cols,
                                    title="选择要标准化的列（按m多选）",
                                    description="选择要标准化的列，按m选择多个，按回车确认",
                                    min_select=1,
                                    max_select=len(numeric_cols)
                                )
                                selected_cols = inter.run()
                                if selected_cols:
                                    way['col'] = selected_cols
        
        # 处理normalized_preprocessing - 让用户选择要归一化的列
        if 'normalized_preprocessing' in params:
            if 'normalized_ways' in params['normalized_preprocessing']:
                for way in params['normalized_preprocessing']['normalized_ways']:
                    if way.get('col') == 'all':
                        if numeric_cols:
                            if auto_test:
                                # 自动测试模式，选择所有数值列
                                way['col'] = numeric_cols
                                ColorPrinter.print_info(f"自动测试模式: 选择所有数值列进行归一化处理: {numeric_cols}")
                            else:
                                ColorPrinter.print_info("\n=== 归一化处理 ===")
                                ColorPrinter.print_info("当前方法: " + way.get('method', 'L2归一化'))
                                inter = interactive(
                                    options=numeric_cols,
                                    title="选择要归一化的列（按m多选）",
                                    description="选择要归一化的列，按m选择多个，按回车确认",
                                    min_select=1,
                                    max_select=len(numeric_cols)
                                )
                                selected_cols = inter.run()
                                if selected_cols:
                                    way['col'] = selected_cols
        
        # 处理time_preprocessing - 让用户选择时间列
        if 'time_preprocessing' in params:
            if params['time_preprocessing'].get('time_col') == 'auto':
                # 尝试自动检测时间列
                time_cols = []
                for col in all_cols:
                    try:
                        pd.to_datetime(data[col], errors='coerce')
                        # 检查该列是否有有效的时间数据
                        if pd.to_datetime(data[col], errors='coerce').notna().any():
                            time_cols.append(col)
                    except:
                        pass
                
                if time_cols:
                    if auto_test:
                        # 自动测试模式，选择第一个时间列
                        params['time_preprocessing']['time_col'] = time_cols[0]
                        ColorPrinter.print_info(f"自动测试模式: 选择第一个时间列: {time_cols[0]}")
                    else:
                        ColorPrinter.print_info("\n=== 时间处理 ===")
                        inter = interactive(
                            options=time_cols,
                            title="选择时间列",
                            description="选择包含时间信息的列",
                            min_select=1,
                            max_select=1
                        )
                        selected_cols = inter.run()
                        if selected_cols:
                            params['time_preprocessing']['time_col'] = selected_cols[0] if selected_cols else None
                else:
                    # 没有检测到时间列
                    ColorPrinter.print_warning("未检测到时间列")
        
        # 处理relative_preprocessing - 让用户选择特征工程相关的列
        if 'relative_preprocessing' in params:
            if params['relative_preprocessing'].get('linear_cols') == 'all':
                if numeric_cols:
                    if auto_test:
                        # 自动测试模式，选择所有数值列
                        params['relative_preprocessing']['linear_cols'] = numeric_cols
                        params['relative_preprocessing']['feature_cols'] = numeric_cols
                        params['relative_preprocessing']['pca_cols'] = numeric_cols
                        ColorPrinter.print_info(f"自动测试模式: 选择所有数值列进行特征工程: {numeric_cols}")
                    else:
                        ColorPrinter.print_info("\n=== 特征工程（线性组合） ===")
                        inter = interactive(
                            options=numeric_cols,
                            title="选择用于线性组合的列（按m多选）",
                            description="选择要计算线性组合特征的列，按m选择多个，按回车确认",
                            min_select=1,
                            max_select=len(numeric_cols)
                        )
                        selected_cols = inter.run()
                        if selected_cols:
                            params['relative_preprocessing']['linear_cols'] = selected_cols
    
    # 配置后展示用户选择的路线
    ColorPrinter.print_info("\n配置后 - 您选择的路线详细信息:")
    
    if route_number in ROUTES:
        route_info = ROUTES[route_number]
        ColorPrinter.print_info(f"\n{route_number}. {route_info['name']}")
        ColorPrinter.print_info(f"   📝 描述: {route_info['description']}")
        ColorPrinter.print_info(f"   🔧 工具: {', '.join(route_info['tools'])}")
        ColorPrinter.print_info(f"   🎯 适用: {route_info['suitable_for']}")
    
    return params


def show_available_routes():
    """
    显示所有可用的预定义路线
    
    Returns:
        list: 路线列表，包含所有可用的预定义路线编号和名称
    """
    routes = [
        "1: 基础路线 - 适合大多数数据集",
        "2: 股票数据路线 - 专为股票数据优化",
        "3: 文本数据路线 - 包含文本处理",
        "4: 时间序列路线 - 强化时间特征提取",
        "5: 完整路线 - 包含所有预处理步骤",
        "6: 轻量级路线 - 快速处理小数据集",
        "7: 异常检测路线 - 专注于异常值处理",
        "8: 特征增强路线 - 强化特征工程",
        "9: 分类数据路线 - 适合分类任务",
        "10: 回归数据路线 - 适合回归任务",
        "11: 中文文本路线 - 专为中文文本数据优化",
        "12: 不平衡数据路线 - 处理类别不平衡的数据集",
        "13: 高维数据路线 - 处理特征维度较高的数据集",
        "14: 金融数据路线 - 专为金融数据优化",
        "15: 医疗数据路线 - 适合医疗数据集的处理",
        "16: 电商数据路线 - 专为电商数据优化",
        "17: 社交媒体数据路线 - 处理社交媒体数据",
        "18: 传感器数据路线 - 处理传感器数据",
        "19: 多模态数据路线 - 处理包含多种类型数据的数据集",
        "20: 实时数据路线 - 适合实时数据流处理"
    ]
    
    ColorPrinter.print_info("\n可用的预定义路线:")
    for route in routes:
        ColorPrinter.print_info(route)
    
    return routes


def show_routes_visual():
    """
    可视化显示所有可用的预定义路线及其详细简介
    """
    ColorPrinter.print_info("\n" + "=" * 80)
    ColorPrinter.print_info("📋 预定义特征工程路线详细说明")
    ColorPrinter.print_info("=" * 80)
    
    for route_number, route_info in ROUTES.items():
        ColorPrinter.print_info(f"\n{route_number}. {route_info['name']}")
        ColorPrinter.print_info(f"   📝 描述: {route_info['description']}")
        ColorPrinter.print_info(f"   🔧 工具: {', '.join(route_info['tools'])}")
        ColorPrinter.print_info(f"   🎯 适用: {route_info['suitable_for']}")
    
    ColorPrinter.print_info("\n" + "=" * 80)



import pandas as pd
import numpy as np
from scipy import stats

import sys
import os

current_dir = os.path.dirname(os.path.abspath(__file__))
root_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(current_dir)))))
if root_dir not in sys.path:
    sys.path.append(root_dir)

from myffe.tools.interactive import interactive
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.visualize_route import visualize_route
from myffe.tools.interactive_utils import get_interactive_choice
from myffe.route.tools_py.outlier_detector import is_numeric_columns, detect_outliers_iqr, detect_outliers_zscore

# 异常值分析主函数

def analyze_outlier_distribution(data, auto_test=False):
    """分析数据的异常值分布，为异常值处理提供参数
    
    Args:
        data: pandas DataFrame，输入数据
        auto_test: bool，是否自动测试模式，默认为 False
    
    Returns:
        dict: 包含异常值处理参数的字典
    """
    # 初始化异常值处理参数
    outlier_params = {
        'outlier_choice': 'multivariate',
        'row_drop_thres': 3.0,
        'auto_drop_row': False,
        'outlier_action': 'drop',
        'contamination': 0.1,
        'specify_auto_drop_row_extr': None,
        'specify_row': None
    }

    # 筛选数值列
    numeric_columns = []
    for col in data.columns:
        col_data = data[col].astype(str).tolist()
        if is_numeric_columns(col_data):
            numeric_columns.append(col)

    # 检查是否有数值列
    if not numeric_columns:
        ColorPrinter.print_warning("数据中没有数值列")
        return {'outlier_preprocessing': 'skip'}

#region 检测异常值，并且得到异常值列
    has_outliers = False
    outlier_columns = []
    for col in numeric_columns:  # 遍历数值列并检测异常值
        try:
            data[col] = pd.to_numeric(data[col])
            if detect_outliers_iqr(data, col) or detect_outliers_zscore(data, col):
                has_outliers = True
                outlier_columns.append(col)
        except ValueError:
            continue
#endregion

    #region 检查是否有异常值
    if not has_outliers:
        ColorPrinter.print_info("数据中未检测到异常值")
        return {'outlier_preprocessing': 'skip'}
    ColorPrinter.print_success(f"检测到异常值列：{outlier_columns}")
    #endregion 检查是否有异常值

    # 获取智能推荐选择
    inter_outlier = get_interactive_choice(
        title="智能推荐？",
        description="选择'ok'将自动推荐最佳异常值处理方法，选择'no'需要手动选择",
        auto_test=auto_test
    )

    # 智能推荐模式
    if inter_outlier:
        data_size = len(data)
        num_features = len(numeric_columns)

        # 根据数据特征选择最佳异常值检测方法
        if num_features == 1:
            outlier_choice = 'thres_sigma_auto'
        elif num_features >= 2 and data_size > 100:
            outlier_choice = 'multivariate'
        elif data_size > 1000:
            outlier_choice = 'boxplot'
        else:
            outlier_choice = 'isolation_forest'

        outlier_params['outlier_choice'] = outlier_choice
        outlier_params['outlier_action'] = 'drop'

        # 根据选择的方法设置相应参数
        if outlier_choice == 'thres_sigma_auto':
            outlier_params['row_drop_thres'] = 3.0
            outlier_params['specify_auto_drop_row_extr'] = 'auto'
        elif outlier_choice in ['isolation_forest', 'multivariate']:
            outlier_params['contamination'] = 0.1
        elif outlier_choice == 'boxplot':
            pass

        ColorPrinter.print_info(f"智能推荐方法：{outlier_choice}")
        ColorPrinter.print_info(f"智能推荐动作：{outlier_params['outlier_action']}")
    else:
        # 手动选择模式
        outlier_choices = [
            'thres_sigma_auto',
            'boxplot',
            'isolation_forest',
            'multivariate',
            'skip'
        ]
        selected = interactive(
            outlier_choices,
            title="请选择异常值处理方法",
            description="thres_sigma_auto:基于标准差, boxplot:箱线图, isolation_forest:孤立森林, multivariate:多变量检测, skip:跳过",
            min_select=1, max_select=1
        ).run()

        outlier_params['outlier_choice'] = selected[0]

    outlier_choice = outlier_params['outlier_choice']

#region 处理跳过选择
    if outlier_choice == 'skip':
        ColorPrinter.print_info("跳过异常值处理")
        return {'outlier_preprocessing': 'skip'}
#endregion 
    # 手动设置参数
    if not inter_outlier:
#region 选择异常值处理动作: outlier_action
        outlier_action_choices = ['drop', 'replace']
        selected_action = interactive(
            outlier_action_choices,
            title="请选择异常值处理动作",
            description="drop:删除异常值行, replace:替换异常值",
            min_select=1, max_select=1
        ).run()
        outlier_params['outlier_action'] = selected_action[0]
#endregion 选择异常值处理动作
#region 设置标准差倍数阈值: row_drop_thres
        if outlier_choice == 'thres_sigma_auto':
            threshold_options = ['1.0', '2.0', '2.5', '3.0', '3.5', '4.0']
            selected_thres = interactive(
                threshold_options,
                title="请选择标准差倍数阈值",
                description="用于判断异常值的阈值倍数，值越小检测越严格",
                min_select=1, max_select=1
            ).run()
            outlier_params['row_drop_thres'] = float(selected_thres[0])
        else:
            outlier_params['row_drop_thres'] = 3.0
#region 选择是否指定检测列: specify_auto_drop_row_extr
        # 选择是否指定检测列
        ColorPrinter.print_info("是否指定检测列？")
        specify_col_choice = interactive(
            ['auto', '手动选择', '不指定'],
            title="请选择",
            description="auto:对所有数值列检测, 手动选择:选择特定列, 不指定:不指定列",
            min_select=1, max_select=1
        ).run()

        if specify_col_choice[0] == '手动选择':
            selected_cols = interactive(
                numeric_columns,
                title="请选择要检测的列",
                description="选择要进行异常值检测的列，可多选",
                min_select=1, max_select=len(numeric_columns)
            ).run()
            outlier_params['specify_auto_drop_row_extr'] = selected_cols
        elif specify_col_choice[0] == 'auto':
            outlier_params['specify_auto_drop_row_extr'] = 'auto'
        else:
            outlier_params['specify_auto_drop_row_extr'] = None
#endregion 选择是否指定检测列
#region 设置异常值比例: contamination
        if outlier_choice in ['isolation_forest', 'multivariate']:
            contamination_options = ['0.01', '0.05', '0.1', '0.15', '0.2', '0.3']
            selected_cont = interactive(
                contamination_options,
                title="请选择异常值比例",
                description="异常值占总样本的比例估计",
                min_select=1, max_select=1
            ).run()
            outlier_params['contamination'] = float(selected_cont[0])
#endregion 
    # 生成结果并可视化
    result = {'outlier_preprocessing': outlier_params}
    visualize_route(result)
    return result

def outlier_py(data, auto_test=False):
    return analyze_outlier_distribution(data, auto_test=auto_test)

if __name__ == "__main__":
    import pandas as pd
    import numpy as np

    np.random.seed(42)
    data = pd.DataFrame({
        'feature1': np.random.normal(50, 10, 100),
        'feature2': np.random.normal(100, 20, 100),
        'feature3': np.random.normal(200, 30, 100),
        'label': ['A'] * 50 + ['B'] * 50
    })

    data.loc[0, 'feature1'] = 500
    data.loc[1, 'feature2'] = 800
    data.loc[2, 'feature3'] = 1200

    ColorPrinter.print_info("测试数据:")
    ColorPrinter.print_any(data.head(), color='white')

    ColorPrinter.print_info("\n测试异常值处理:")
    result = outlier_py(data)
    ColorPrinter.print_success("处理结果:")
    ColorPrinter.print_any(result, color='white')



import pandas as pd
import numpy as np
from scipy import stats
from sklearn.preprocessing import StandardScaler
from myffe.tools.color_printer import ColorPrinter


def evaluate_dataset(df, original_df=None, label_column=None):
    """
    Evaluate dataset metrics and calculate comprehensive score
    
    Args:
        df: Processed dataset (pandas DataFrame)
        original_df: Original dataset (pandas DataFrame), used for comparing processing effects
        label_column: Label column name, used for evaluating class balance
    
    Returns:
        dict: Dictionary containing various metric scores and comprehensive score
    """
    scores = {}
    
    # 1. Data completeness metrics
    # Missing value ratio
    total_cells = df.shape[0] * df.shape[1]
    missing_cells = df.isnull().sum().sum()
    missing_ratio = missing_cells / total_cells
    # Lower missing ratio gets higher score
    scores['missing_ratio_score'] = max(0, 100 - missing_ratio * 100)
    
    # Number of rows
    if original_df is not None:
        row_keeping_ratio = len(df) / len(original_df)
        # Higher row keeping ratio gets higher score
        scores['row_keeping_score'] = min(100, row_keeping_ratio * 100)
    else:
        # If no original data, score based on absolute row count
        if len(df) >= 10000:
            scores['row_keeping_score'] = 100
        elif len(df) >= 1000:
            scores['row_keeping_score'] = 80
        elif len(df) >= 100:
            scores['row_keeping_score'] = 60
        else:
            scores['row_keeping_score'] = 40
    
    # Number of columns
    scores['column_count_score'] = min(100, df.shape[1] * 5)  # 5 points per column, max 100 points
    
    # 2. Data quality metrics
    # Outlier handling effect (using IQR method to detect outliers)
    numeric_cols = df.select_dtypes(include=[np.number]).columns
    outlier_count = 0
    total_numeric_cells = 0
    
    for col in numeric_cols:
        if len(df[col].dropna()) > 0:
            Q1 = df[col].quantile(0.25)
            Q3 = df[col].quantile(0.75)
            IQR = Q3 - Q1
            lower_bound = Q1 - 1.5 * IQR
            upper_bound = Q3 + 1.5 * IQR
            outlier_count += ((df[col] < lower_bound) | (df[col] > upper_bound)).sum()
            total_numeric_cells += len(df[col].dropna())
    
    if total_numeric_cells > 0:
        outlier_ratio = outlier_count / total_numeric_cells
        # Lower outlier ratio gets higher score
        scores['outlier_score'] = max(0, 100 - outlier_ratio * 100)
    else:
        scores['outlier_score'] = 100
    
    # Data consistency (data type consistency)
    # Check if each column has consistent data types
    consistency_score = 100
    for col in df.columns:
        # Calculate number of data types for non-null values
        non_null_values = df[col].dropna()
        if len(non_null_values) > 0:
            # Check if all values are the same type
            types = non_null_values.apply(type).nunique()
            if types > 1:
                consistency_score -= 10  # Deduct 10 points for each inconsistent type
    scores['consistency_score'] = max(0, consistency_score)
    
    # 3. Feature quality metrics
    # Feature diversity (calculate unique value ratio for each column)
    diversity_scores = []
    for col in df.columns:
        if len(df[col].dropna()) > 0:
            unique_ratio = df[col].nunique() / len(df[col].dropna())
            diversity_scores.append(min(1, unique_ratio) * 100)
    
    if diversity_scores:
        scores['diversity_score'] = np.mean(diversity_scores)
    else:
        scores['diversity_score'] = 0
    
    # Feature correlation (calculate correlation coefficients between numeric features)
    if len(numeric_cols) >= 2:
        corr_matrix = df[numeric_cols].corr()
        # Exclude diagonal (self-correlation)
        upper_tri = corr_matrix.where(np.triu(np.ones(corr_matrix.shape), k=1).astype(bool))
        # Calculate average absolute correlation coefficient
        mean_abs_corr = upper_tri.stack().abs().mean()
        # Lower correlation is better (avoid multicollinearity)
        scores['correlation_score'] = max(0, 100 - mean_abs_corr * 100)
    else:
        scores['correlation_score'] = 100
    
    # Feature information content (calculate variance of numeric features)
    variance_scores = []
    for col in numeric_cols:
        if len(df[col].dropna()) > 0:
            var = df[col].var()
            # Normalize variance score
            if var > 0:
                variance_scores.append(min(100, var * 10))
            else:
                variance_scores.append(0)
    
    if variance_scores:
        scores['information_score'] = np.mean(variance_scores)
    else:
        scores['information_score'] = 0
    
    # 4. Data balance metrics
    # Class balance (if there is a label column)
    if label_column and label_column in df.columns:
        label_counts = df[label_column].value_counts()
        if len(label_counts) > 1:
            # Use Gini coefficient to evaluate balance
            total = len(df[label_column])
            proportions = label_counts / total
            gini = 1 - sum(proportions ** 2)
            # Gini coefficient closer to 1 (perfect balance) gets higher score
            scores['balance_score'] = gini * 100
        else:
            scores['balance_score'] = 50  # Only one class, give medium score
    else:
        scores['balance_score'] = 100  # No label column, give full score
    
    # 5. Feature range metrics
    # Check if numeric feature ranges are reasonable
    range_scores = []
    for col in numeric_cols:
        if len(df[col].dropna()) > 0:
            col_min = df[col].min()
            col_max = df[col].max()
            # Check for extreme values
            if col_max - col_min > 0:
                # Normalize range score
                range_score = min(100, 100 - abs(np.log10(col_max - col_min)) * 10)
                range_scores.append(range_score)
            else:
                range_scores.append(50)  # All values the same, give medium score
    
    if range_scores:
        scores['range_score'] = np.mean(range_scores)
    else:
        scores['range_score'] = 100
    
    # 6. Model compatibility metrics
    # Feature types
    # Check for feature types that are not suitable for models
    incompatible_types = 0
    for col in df.columns:
        col_dtype = df[col].dtype
        if col_dtype == 'object':
            # Check if it's a pure string column
            if all(isinstance(x, str) for x in df[col].dropna()):
                incompatible_types += 1
    
    total_cols = len(df.columns)
    if total_cols > 0:
        compatibility_score = 100 - (incompatible_types / total_cols) * 50  # Deduct 50 points for each incompatible type
        scores['compatibility_score'] = max(0, compatibility_score)
    else:
        scores['compatibility_score'] = 100
    
    # Calculate comprehensive score
    # Assign weights to different metrics
    weights = {
        'missing_ratio_score': 0.15,
        'row_keeping_score': 0.1,
        'column_count_score': 0.1,
        'outlier_score': 0.1,
        'consistency_score': 0.1,
        'diversity_score': 0.1,
        'correlation_score': 0.05,
        'information_score': 0.1,
        'balance_score': 0.1,
        'range_score': 0.05,
        'compatibility_score': 0.05
    }
    
    # Calculate weighted average
    total_score = sum(scores[key] * weights[key] for key in scores)
    
    # Return result
    result = {
        'detailed_scores': scores,
        'total_score': round(total_score, 2),
        'data_shape': df.shape
    }
    
    return result


def print_evaluation_result(result):
    """
    Print evaluation result
    
    Args:
        result: Result dictionary returned by evaluate_dataset function
    """
    ColorPrinter.print_info("=" * 80)
    ColorPrinter.print_info("Dataset Evaluation Result")
    ColorPrinter.print_info("=" * 80)
    ColorPrinter.print_info(f"Total score: {result['total_score']}/100")
    ColorPrinter.print_info(f"Data shape: {result['data_shape']}")
    ColorPrinter.print_info("\nDetailed metric scores:")
    ColorPrinter.print_info("-" * 80)
    
    # Group by category
    categories = {
        'Data completeness': ['missing_ratio_score', 'row_keeping_score', 'column_count_score'],
        'Data quality': ['outlier_score', 'consistency_score'],
        'Feature quality': ['diversity_score', 'correlation_score', 'information_score'],
        'Data balance': ['balance_score', 'range_score'],
        'Model compatibility': ['compatibility_score']
    }
    
    for category, metrics in categories.items():
        ColorPrinter.print_info(f"\n{category}:")
        for metric in metrics:
            if metric in result['detailed_scores']:
                score = result['detailed_scores'][metric]
                ColorPrinter.print_info(f"  • {metric.replace('_score', '')}: {score:.2f}/100")
    
    ColorPrinter.print_info("=" * 80)


if __name__ == "__main__":
    # 测试代码
    # 创建示例数据
    data = {
        'age': [25, 30, 35, 40, 45, 50, 55, 60, 65, 70],
        'income': [50000, 60000, 70000, 80000, 90000, 100000, 110000, 120000, 130000, 140000],
        'score': [85, 90, 75, 80, 95, 70, 85, 90, 80, 75],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    }
    
    df = pd.DataFrame(data)
    
    # 评估数据集
    result = evaluate_dataset(df, label_column='label')
    
    # 打印结果
    print_evaluation_result(result)



import sys
import os

# 添加项目根目录到Python路径
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..')))

from myffe.tools.Loggerv1_1 import log_class
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.evaluation_decorator import with_evaluation
from myffe.tools.data_inspector import inspect_data
from myffe.tools.param_utils import convert_to_list
from myffe.preprocessing.function.relativefunction import linear_plus, feature_plus, feature_importance, pca_lda_kernel

@log_class(type='use')
class RelativePreprocessing:
    def __init__(self, shortcut_relative=None):
        self.shortcut_relative = shortcut_relative

    @with_evaluation
    def __call__(self, data, data_label=None):
        self.data_label = data_label
        
        self.data = data.copy()
        ColorPrinter.print_progress('开始相关性分析和特征工程')
        
        # 使用 pandas 处理数据
        ColorPrinter.print_info('使用 pandas 处理')
        processed_data = self._process_with_pandas(data.copy())
        
        # 返回完整数据
        return processed_data
    
    def _process_with_pandas(self, data):
        """使用pandas处理数据"""
        # 统一使用一个处理方法，不再区分普通和高级模式
        data = self.relative_normal_switch(data)
        return data



    def relative_normal_switch(self, data):
        # 直接检查各个方法的参数是否存在，决定执行哪些特征工程方法
        
        try:
            # 线性特征组合
            linear_cols = self.shortcut_relative.get('linear_cols')
            if linear_cols:
                threshold = self.shortcut_relative.get('linear_threshold', 0.75)
                data = linear_plus(data, linear_cols, self.data_label, threshold)

            # 非线性特征构建
            feature_cols = self.shortcut_relative.get('feature_cols')
            if feature_cols:
                threshold = self.shortcut_relative.get('feature_threshold', 0.75)
                relation_func = self.shortcut_relative.get('relation_func', 'func1')
                data = feature_plus(data, feature_cols, self.data_label, threshold, relation_func)

            # 特征重要性评估
            feature_cols = self.shortcut_relative.get('feature_cols')
            if feature_cols:
                n_features = self.shortcut_relative.get('n_features', 10)
                data = feature_importance(data, feature_cols, self.data_label, n_features)

            # 降维处理
            pca_cols = self.shortcut_relative.get('pca_cols')
            if pca_cols:
                n_components = self.shortcut_relative.get('n_components', 10)
                kernel_selection = self.shortcut_relative.get('kernel_selection', 'pca')
                data = pca_lda_kernel(data, pca_cols, self.data_label, n_components, kernel_selection)

        except Exception as e:
            ColorPrinter.print_error(f'特征工程处理出错: {e}')

        return data




"""
Parameter documentation:

shortcut_relative parameter dictionary:
    - linear_cols: Columns for linear combination
      Type: list of column names
      Default: None (must be specified in linear_plus mode)
      Description: Select columns from numeric columns for linear combination
    
    - linear_threshold: Correlation coefficient threshold for linear combination
      Type: float
      Optional values: 0.0 - 1.0
      Default: None (must be specified in linear_plus mode)
      Description: Columns with correlation coefficient greater than this value will be linearly combined
    
    - feature_cols: Columns for feature construction
      Type: list of column names
      Default: None (must be specified in feature_plus mode)
      Description: Select columns from numeric columns for feature construction
    
    - feature_threshold: R² threshold for feature construction
      Type: float
      Optional values: 0.0 - 1.0
      Default: None (must be specified in feature_plus mode)
      Description: New features will only be added if the fitted R² value is greater than this threshold
    
    - relation_func: Function type for feature construction
      Type: string
      Optional values: 'func1', 'func2', 'func3', 'func4', 'func5'
      Default: 'func1'
      Description: Select function type for fitting:
        - 'func1': Quadratic function a*x^2 + b*x + c
        - 'func2': Cubic function a*x^3 + b*x^2 + c*x + d
        - 'func3': Exponential function a*exp(b*x) + c
        - 'func4': Logarithmic function a*log(x) + b
        - 'func5': Square root function a*sqrt(x) + b
      Note: This parameter must be specified in feature_plus mode
    
    - pca_cols: Columns for dimensionality reduction
      Type: list of column names
      Default: None (must be specified in pca_lda_kernel mode)
      Description: Select columns from numeric columns for dimensionality reduction
    
    - n_components: Number of dimensions after dimensionality reduction
      Type: integer
      Optional values: >= 1
      Default: None (must be specified in pca_lda_kernel mode)
      Description: Number of features after dimensionality reduction
    
    - kernel_selection: Dimensionality reduction method selection
      Type: string
      Optional values: 'pca', 'lda', 'kernel'
      Default: None (must be specified in pca_lda_kernel mode)
      Description: 
        - 'pca': Principal Component Analysis
        - 'lda': Linear Discriminant Analysis (requires label column)
        - 'kernel': Kernel Principal Component Analysis
    
    - n_features: Number of features for importance evaluation
      Type: integer
      Optional values: >= 1
      Default: 10
      Description: Number of features to retain after feature importance evaluation (only used in feature_importance mode)

Usage example:
    # Example 1: Using quadratic function for feature construction
    relative_params = {
        'feature_cols': ['col1', 'col2', 'col3'],
        'feature_threshold': 0.75,
        'relation_func': 'func1'
    }
    relative_processor = RelativePreprocessing(shortcut_relative=relative_params)
    processed_data = relative_processor(data, 'label')
    
    # Example 2: Using cubic function for feature construction
    relative_params = {
        'feature_cols': ['col1', 'col2', 'col3'],
        'feature_threshold': 0.8,
        'relation_func': 'func2'
    }
    
    # Example 3: Using exponential function for feature construction
    relative_params = {
        'feature_cols': ['col1', 'col2'],
        'feature_threshold': 0.7,
        'relation_func': 'func3'
    }
    
    # Example 4: Using logarithmic function for feature construction
    relative_params = {
        'feature_cols': ['col1', 'col2'],
        'feature_threshold': 0.7,
        'relation_func': 'func4'
    }
    
    # Example 5: Using square root function for feature construction
    relative_params = {
        'feature_cols': ['col1', 'col2'],
        'feature_threshold': 0.7,
        'relation_func': 'func5'
    }
    
    # Example 6: Using feature importance evaluation
    relative_params = {
        'feature_cols': ['col1', 'col2', 'col3', 'col4', 'col5'],
        'n_features': 3
    }
    relative_processor = RelativePreprocessing(shortcut_relative=relative_params)
    processed_data = relative_processor(data, 'label')
"""


if __name__ == "__main__":
    import pandas as pd
    import numpy as np
    
    print("=" * 60)
    print("相关性分析和特征工程测试程序")
    print("=" * 60)
    
    # 创建测试数据
    np.random.seed(42)
    test_data = pd.DataFrame({
        'feature1': np.random.normal(50, 10, 50),
        'feature2': np.random.normal(100, 20, 50),
        'feature3': np.random.normal(200, 30, 50),
        'feature4': np.random.normal(300, 40, 50),
        'label': ['A'] * 25 + ['B'] * 25
    })
    
    # 添加一些相关性
    test_data['feature5'] = test_data['feature1'] * 2 + np.random.normal(0, 5, 50)
    test_data['feature6'] = test_data['feature2'] * 1.5 + np.random.normal(0, 5, 50)
    
    print("\n原始数据:")
    print(test_data.head(10))
    print(f"\n数据形状: {test_data.shape}")
    
    # 测试不同的特征工程方式
    test_cases = [
        {
            'params': {
                'linear_cols': ['feature1', 'feature2', 'feature3', 'feature4', 'feature5', 'feature6'],
                'linear_threshold': 0.8
            },
            'description': '线性特征组合'
        },
        {
            'params': {
                'feature_cols': ['feature1', 'feature2', 'feature3'],
                'feature_threshold': 0.8,
                'relation_func': 'func1'
            },
            'description': '非线性特征构建-二次函数'
        },
        {
            'params': {
                'feature_cols': ['feature1', 'feature2', 'feature3'],
                'feature_threshold': 0.8,
                'relation_func': 'func2'
            },
            'description': '非线性特征构建-三次函数'
        },
        {
            'params': {
                'feature_cols': ['feature1', 'feature2'],
                'feature_threshold': 0.7,
                'relation_func': 'func3'
            },
            'description': '非线性特征构建-指数函数'
        },
        {
            'params': {
                'pca_cols': ['feature1', 'feature2', 'feature3', 'feature4', 'feature5', 'feature6'],
                'n_components': 3,
                'kernel_selection': 'pca'
            },
            'description': 'PCA降维'
        },
        {
            'params': {},
            'description': '跳过特征工程'
        }
    ]
    
    for i, test_case in enumerate(test_cases, 1):
        print(f"\n{'=' * 60}")
        print(f"测试 {i}: {test_case['description']}")
        print('=' * 60)
        
        relative_processor = RelativePreprocessing(shortcut_relative=test_case['params'])
        try:
            processed_data = relative_processor(test_data.copy(), 'label')
            
            print("\n处理后的数据形状:")
            print(f"原始数据: {test_data.shape}")
            print(f"处理后数据: {processed_data.shape}")
            
            if processed_data.shape[1] != test_data.shape[1]:
                print("\n新增/删除的列:")
                original_cols = set(test_data.columns)
                processed_cols = set(processed_data.columns)
                added_cols = processed_cols - original_cols
                removed_cols = original_cols - processed_cols
                if added_cols:
                    print(f"新增列: {added_cols}")
                if removed_cols:
                    print(f"删除列: {removed_cols}")
        except Exception as e:
            print(f"\n处理出错: {e}")
    
    print("\n" + "=" * 60)
    print("测试完成！")
    print("=" * 60)
    
    # 深度渗透测试套件
    print(f"\n{'=' * 80}")
    print("深度渗透测试套件")
    print('=' * 80)
    
    # 创建更大的测试数据
    np.random.seed(42)
    large_test_data = pd.DataFrame({
        'feature1': np.random.normal(50, 10, 1000),
        'feature2': np.random.normal(100, 20, 1000),
        'feature3': np.random.normal(200, 30, 1000),
        'feature4': np.random.normal(300, 40, 1000),
        'feature5': np.random.normal(400, 50, 1000),
        'feature6': np.random.normal(500, 60, 1000),
        'feature7': np.random.normal(600, 70, 1000),
        'feature8': np.random.normal(700, 80, 1000),
        'feature9': np.random.normal(800, 90, 1000),
        'feature10': np.random.normal(900, 100, 1000),
        'label': np.random.choice(['A', 'B', 'C'], 1000)
    })
    
    # 添加一些强相关性
    large_test_data['feature11'] = large_test_data['feature1'] * 2 + np.random.normal(0, 2, 1000)
    large_test_data['feature12'] = large_test_data['feature2'] * 3 + np.random.normal(0, 3, 1000)
    large_test_data['feature13'] = large_test_data['feature3'] * 1.5 + np.random.normal(0, 1.5, 1000)
    
    # 测试1: 边界条件测试 - 空参数
    print(f"\n{'-' * 60}")
    print("测试1: 边界条件测试 - 空参数")
    print('-' * 60)
    
    relative_processor_1 = RelativePreprocessing(shortcut_relative={})
    result_1 = relative_processor_1(large_test_data.copy(), 'label')
    
    print(f"\n数据形状变化: {large_test_data.shape} -> {result_1.shape}")
    print("期望：数据保持不变")
    
    # 测试2: 边界条件测试 - 不存在的列
    print(f"\n{'-' * 60}")
    print("测试2: 边界条件测试 - 不存在的列")
    print('-' * 60)
    
    params_nonexistent = {
        'linear_cols': ['nonexistent_col', 'feature1', 'feature2'],
        'linear_threshold': 0.8
    }
    relative_processor_2 = RelativePreprocessing(shortcut_relative=params_nonexistent)
    result_2 = relative_processor_2(large_test_data.copy(), 'label')
    
    print(f"\n数据形状变化: {large_test_data.shape} -> {result_2.shape}")
    print("期望：只处理存在的列")
    
    # 测试3: 边界条件测试 - 列数不足
    print(f"\n{'-' * 60}")
    print("测试3: 边界条件测试 - 列数不足")
    print('-' * 60)
    
    params_few_cols = {
        'linear_cols': ['feature1'],
        'linear_threshold': 0.8
    }
    relative_processor_3 = RelativePreprocessing(shortcut_relative=params_few_cols)
    result_3 = relative_processor_3(large_test_data.copy(), 'label')
    
    print(f"\n数据形状变化: {large_test_data.shape} -> {result_3.shape}")
    print("期望：数据保持不变，至少需要2列")
    
    # 测试4: 边界条件测试 - 阈值边界值
    print(f"\n{'-' * 60}")
    print("测试4: 边界条件测试 - 阈值边界值")
    print('-' * 60)
    
    test_cases_threshold = [
        {'threshold': 0.0, 'desc': '阈值为0'},
        {'threshold': 1.0, 'desc': '阈值为1'},
    ]
    
    for tc in test_cases_threshold:
        params_threshold = {
            'linear_cols': ['feature1', 'feature2', 'feature3', 'feature11', 'feature12'],
            'linear_threshold': tc['threshold']
        }
        relative_processor = RelativePreprocessing(shortcut_relative=params_threshold)
        result = relative_processor(large_test_data.copy(), 'label')
        print(f"\n{tc['desc']}:")
        print(f"  数据形状变化: {large_test_data.shape} -> {result.shape}")
    
    # 测试5: 边界条件测试 - 只有一个数值列
    print(f"\n{'-' * 60}")
    print("测试5: 边界条件测试 - 只有一个数值列")
    print('-' * 60)
    
    data_single_numeric = pd.DataFrame({
        'numeric_col': np.random.normal(50, 10, 100),
        'str_col': ['a'] * 100,
        'label': ['A'] * 50 + ['B'] * 50
    })
    
    params_single_numeric = {
        'linear_cols': ['numeric_col', 'str_col'],
        'linear_threshold': 0.8
    }
    relative_processor_5 = RelativePreprocessing(shortcut_relative=params_single_numeric)
    result_5 = relative_processor_5(data_single_numeric.copy(), 'label')
    
    print(f"\n数据形状变化: {data_single_numeric.shape} -> {result_5.shape}")
    print("期望：数据保持不变，需要至少2个数值列")
    
    # 测试6: 边界条件测试 - 没有数值列
    print(f"\n{'-' * 60}")
    print("测试6: 边界条件测试 - 没有数值列")
    print('-' * 60)
    
    data_no_numeric = pd.DataFrame({
        'str_col1': ['a', 'b', 'c', 'd', 'e'] * 20,
        'str_col2': ['x', 'y', 'z', 'w', 'v'] * 20,
        'label': ['A'] * 50 + ['B'] * 50
    })
    
    params_no_numeric = {
        'linear_cols': ['str_col1', 'str_col2'],
        'linear_threshold': 0.8
    }
    relative_processor_6 = RelativePreprocessing(shortcut_relative=params_no_numeric)
    result_6 = relative_processor_6(data_no_numeric.copy(), 'label')
    
    print(f"\n数据形状变化: {data_no_numeric.shape} -> {result_6.shape}")
    print("期望：数据保持不变")
    
    # 测试7: 边界条件测试 - 空数据
    print(f"\n{'-' * 60}")
    print("测试7: 边界条件测试 - 空数据")
    print('-' * 60)
    
    empty_data = pd.DataFrame({
        'feature1': [],
        'feature2': [],
        'label': []
    })
    
    params_empty = {
        'linear_cols': ['feature1', 'feature2'],
        'linear_threshold': 0.8
    }
    relative_processor_7 = RelativePreprocessing(shortcut_relative=params_empty)
    result_7 = relative_processor_7(empty_data.copy(), 'label')
    
    print(f"\n数据形状变化: {empty_data.shape} -> {result_7.shape}")
    print("期望：空数据保持不变")
    
    # 测试8: 组合测试 - 所有函数类型测试
    print(f"\n{'-' * 60}")
    print("测试8: 组合测试 - 所有函数类型测试")
    print('-' * 60)
    
    funcs = ['func1', 'func2', 'func3', 'func4', 'func5']
    
    for func in funcs:
        params_func = {
            'feature_cols': ['feature1', 'feature2', 'feature3'],
            'feature_threshold': 0.7,
            'relation_func': func
        }
        relative_processor = RelativePreprocessing(shortcut_relative=params_func)
        result = relative_processor(large_test_data.copy(), 'label')
        print(f"\n函数类型 {func}:")
        print(f"  数据形状变化: {large_test_data.shape} -> {result.shape}")
    
    # 测试9: 组合测试 - 降维参数边界值
    print(f"\n{'-' * 60}")
    print("测试9: 组合测试 - 降维参数边界值")
    print('-' * 60)
    
    test_cases_pca = [
        {'n_components': 1, 'desc': '降维到1维'},
        {'n_components': 5, 'desc': '降维到5维'},
        {'n_components': 10, 'desc': '降维到10维'},
    ]
    
    for tc in test_cases_pca:
        params_pca = {
            'pca_cols': ['feature1', 'feature2', 'feature3', 'feature4', 'feature5', 
                        'feature6', 'feature7', 'feature8', 'feature9', 'feature10'],
            'n_components': tc['n_components'],
            'kernel_selection': 'pca'
        }
        relative_processor = RelativePreprocessing(shortcut_relative=params_pca)
        result = relative_processor(large_test_data.copy(), 'label')
        print(f"\n{tc['desc']}:")
        print(f"  数据形状变化: {large_test_data.shape} -> {result.shape}")
    
    # 测试10: 验证结果
    print(f"\n{'-' * 60}")
    print("测试10: 验证结果")
    print('-' * 60)
    
    test_data_verify = pd.DataFrame({
        'x': np.array(range(100)),
        'y': np.array([2 * i + np.random.normal(0, 0.1) for i in range(100)]),
        'label': ['A'] * 50 + ['B'] * 50
    })
    
    params_verify = {
        'linear_cols': ['x', 'y'],
        'linear_threshold': 0.9
    }
    
    relative_processor_10 = RelativePreprocessing(shortcut_relative=params_verify)
    result_10 = relative_processor_10(test_data_verify.copy(), 'label')
    
    print(f"\n原始列: {list(test_data_verify.columns)}")
    print(f"处理后列: {list(result_10.columns)}")
    print("期望：由于x和y高度相关，应该删除其中一列")
    
    print(f"\n{'=' * 80}")
    print("深度渗透测试完成！")
    print('=' * 80)



import pandas as pd
import numpy as np
import sys
import os

# 添加包含MFM模块的目录到Python路径
current_dir = os.path.dirname(os.path.abspath(__file__))
parent_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(current_dir))))
sys.path.append(parent_dir)
print(f"Added parent directory to sys.path: {parent_dir}")

from myffe.tools.interactive import interactive
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.param_utils import convert_to_list
from myffe.route.src.predefined_routes import get_route_params, show_available_routes, show_routes_visual
from myffe.route.src.auto_route import auto_routes

class UnifiedConfigManager:
    """统一配置管理器，结合智能推荐、预制路线和交互式配置"""
    
    def __init__(self):
        self.config = {}
        # 定义所有可用的预制路线
        self.predefined_routes = {
            '基础路线': 1,
            '股票数据路线': 2,
            '文本数据路线': 3,
            '时间序列路线': 4,
            '完整路线': 5,
            '轻量级路线': 6,
            '异常检测路线': 7,
            '特征增强路线': 8,
            '分类数据路线': 9,
            '回归数据路线': 10,
            '中文文本路线': 11,
            '不平衡数据路线': 12,
            '高维数据路线': 13,
            '金融数据路线': 14,
            '医疗数据路线': 15,
            '电商数据路线': 16,
            '社交媒体数据路线': 17,
            '传感器数据路线': 18,
            '多模态数据路线': 19,
            '实时数据路线': 20
        }
    
    def get_config(self, data, mode='interactive'):
        """获取配置参数
        
        Args:
            data: pandas DataFrame，输入数据
            mode: str，配置模式，可选值：'interactive'（交互式）、'auto'（智能推荐）、'predefined'（预制路线）
            
        Returns:
            dict: 配置参数
        """
        if mode == 'auto':
            return self._get_auto_config(data)
        elif mode == 'predefined':
            return self._get_predefined_config()
        else:  # interactive
            return self._get_interactive_config(data)
    
    def _get_auto_config(self, data):
        """获取智能推荐配置"""
        ColorPrinter.print_info("=== 智能推荐模式 ===")
        
        # 创建临时文件来存储数据，因为auto_routes类需要从文件读取
        import tempfile
        import os
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.csv', delete=False) as temp_file:
            data.to_csv(temp_file, index=False)
            temp_file_path = temp_file.name
        
        try:
            # 使用auto_routes类生成推荐路由
            auto_route = auto_routes(ways='pandas', drop_threshold=0.5)
            route = auto_route(temp_file_path)
        finally:
            # 清理临时文件
            if os.path.exists(temp_file_path):
                os.unlink(temp_file_path)   #such as remove the temporary file
        
        return route
    
    def _get_predefined_config(self):
        """获取预制路线配置"""
        ColorPrinter.print_info("=== 预制路线模式 ===")
        
        # 显示可用的预制路线
        route_names = list(self.predefined_routes.keys())
        if not route_names:
            ColorPrinter.print_warning("没有可用的预制路线")
            return {}
        
        # 让用户选择预制路线
        selected_route = interactive(route_names, title="请选择预制路线").run()
        ColorPrinter.print_success(f"已选择预制路线：{selected_route}")
        
        # 获取路线编号
        route_number = self.predefined_routes[selected_route]
        # 使用get_route_params函数获取配置
        config = get_route_params(route_number)
        
        return config
    
    def _get_interactive_config(self, data):
        """获取交互式配置"""
        ColorPrinter.print_info("=== 交互式配置模式 ===")
        
        # 首先让用户选择配置方式
        config_mode = interactive(
            ['智能推荐 + 交互式调整', '预制路线 + 交互式调整', '完全手动配置'],
            title="请选择配置方式"
        ).run()
        
        if config_mode == '智能推荐 + 交互式调整':
            # 先获取智能推荐配置
            auto_config = self._get_auto_config(data)
            # 然后交互式调整
            return self._interactive_adjust(data, auto_config)
        elif config_mode == '预制路线 + 交互式调整':
            # 先获取预制路线配置
            predefined_config = self._get_predefined_config()
            # 然后交互式调整
            return self._interactive_adjust(data, predefined_config)
        else:  # 完全手动配置
            return self._manual_config(data)
    
    def _interactive_adjust(self, data, base_config):
        """交互式调整配置"""
        ColorPrinter.print_info("=== 交互式调整配置 ===")
        ColorPrinter.print_info(f"当前配置：{base_config}")
        
        # 让用户选择是否调整配置
        adjust = interactive(['是', '否'], title="是否调整配置？").run()
        if adjust == '否':
            return base_config
        
        # 让用户选择要调整的预处理步骤
        if base_config:
            available_steps = list(base_config.keys())
            if available_steps:
                selected_steps = interactive(available_steps, title="请选择要调整的预处理步骤").run()
                ColorPrinter.print_success(f"已选择调整步骤：{selected_steps}")
                
                # 这里可以添加具体的步骤调整逻辑
                # 例如，根据选择的步骤类型，提供相应的参数调整选项
                
        ColorPrinter.print_success("配置调整完成")
        return base_config
    
    def _manual_config(self, data):
        """完全手动配置"""
        ColorPrinter.print_info("=== 完全手动配置 ===")
        
        # 让用户选择需要的预处理步骤
        available_tools = [
            'nan_preprocessing', 'outlier_preprocessing', 'str_preprocessing',
            'time_preprocessing', 'relative_preprocessing', 'specify_preprocessing',
            'standard_preprocessing', 'normalized_preprocessing', 'sample_preprocessing'
        ]
        
        selected_tools = interactive(available_tools, title="请选择需要的预处理步骤").run()
        ColorPrinter.print_success(f"已选择预处理步骤：{selected_tools}")
        
        # 为每个选择的步骤配置参数
        manual_config = {}
        for tool in selected_tools:
            ColorPrinter.print_info(f"正在配置 {tool}...")
            # 这里可以添加具体的参数配置逻辑
            # 例如，根据工具类型，提供相应的参数配置选项
            manual_config[tool] = {}
        
        ColorPrinter.print_success("手动配置完成")
        return manual_config

if __name__ == "__main__":
    # 测试代码
    import pandas as pd
    import numpy as np
    
    # 创建测试数据
    np.random.seed(42)
    data = pd.DataFrame({
        'numeric_col1': np.random.normal(100, 10, 20),
        'numeric_col2': np.random.normal(50, 5, 20),
        'categorical_col': np.random.choice(['A', 'B', 'C'], 20),
        'datetime_col': pd.date_range('2020-01-01', periods=20)
    })
    
    # 随机添加缺失值
    for col in data.columns:
        if col != 'datetime_col':
            mask = np.random.choice([True, False], size=20, p=[0.2, 0.8])
            data.loc[mask, col] = np.nan
    
    # 测试统一配置管理器
    manager = UnifiedConfigManager()
    
    # 测试交互式模式
    ColorPrinter.print_info("\n测试交互式模式：")
    interactive_config = manager.get_config(data, mode='interactive')
    ColorPrinter.print_success("交互式配置结果：")
    print(interactive_config)
    
    # 测试智能推荐模式
    ColorPrinter.print_info("\n测试智能推荐模式：")
    auto_config = manager.get_config(data, mode='auto')
    ColorPrinter.print_success("智能推荐配置结果：")
    print(auto_config)
    
    # 测试预制路线模式
    ColorPrinter.print_info("\n测试预制路线模式：")
    predefined_config = manager.get_config(data, mode='predefined')
    ColorPrinter.print_success("预制路线配置结果：")
    print(predefined_config)
